@@ -392,5 +392,7 @@ KeySym XStringToKeysym(char *str);
 #undef USE_REAL_VERTICAL_FONT
 #endif
 #undef NO_DISPLAY_FD
+#define FLICK_SCROLL
+#undef UIWINDOW_SUPPORTS_PREEDITING
 
 #endif
