@@ -3,6 +3,7 @@
 #include "ui_event_source.h"
 
 #include <stdio.h> /* sprintf */
+#include <time.h> /* clock */
 #include <pobl/bl_debug.h>
 #include <pobl/bl_str.h>
 #include <pobl/bl_conf_io.h>
@@ -11,6 +12,7 @@
 
 #include "ui_display.h"
 #include "ui_window.h"
+#include "ui_screen.h"
 
 #define UIWINDOW_OF(term) \
   ((term)->parser->xterm_listener ? (term)->parser->xterm_listener->self : NULL)
@@ -47,13 +49,22 @@ static vt_term_t *get_current_term(void) {
 static void update_ime_text(vt_term_t *term, char *preedit_text, char *commit_text) {
   u_char buf[128];
   size_t len;
+  ui_window_t *win;
+
+  if (vt_term_is_backscrolling(term)) {
+    return;
+  }
+
+  if (preedit_text && cur_preedit_text && strcmp(preedit_text, cur_preedit_text) == 0) {
+    return;
+  }
 
   vt_term_set_config(term, "use_local_echo", "false");
 
-#if 0
-  /* XXX If IM plugin is supported, see update_ime_text() in cocoa.m. */
-  ((ui_screen_t*)uiwindow)->is_preediting = 0;
-#endif
+  if ((win = UIWINDOW_OF(term))) {
+    /* XXX If IM plugin is supported, see update_ime_text() in cocoa.m. */
+    ((ui_screen_t*)win)->is_preediting = 0;
+  }
 
   if (!utf8_parser && !(utf8_parser = vt_char_encoding_parser_new(VT_UTF8))) {
     return;
@@ -62,20 +73,14 @@ static void update_ime_text(vt_term_t *term, char *preedit_text, char *commit_te
   (*utf8_parser->init)(utf8_parser);
 
   if (preedit_text) {
-    ui_window_t *win;
-
     if (*preedit_text == '\0') {
       preedit_text = NULL;
     } else {
-      if (bl_compare_str(preedit_text, cur_preedit_text) == 0) {
-        return;
+      if (win) {
+        /* XXX If IM plugin is supported, see update_ime_text() in cocoa.m. */
+        ((ui_screen_t*)win)->is_preediting = 1; /* Hide cursor (stop blinking) */
       }
 
-#if 0
-      /* XXX If IM plugin is supported, see update_ime_text() in cocoa.m. */
-      ((ui_screen_t*)uiwindow)->is_preediting = 1; /* Hide cursor */
-#endif
-
       vt_term_set_config(term, "use_local_echo", "true");
 
       (*utf8_parser->set_str)(utf8_parser, preedit_text, strlen(preedit_text));
@@ -85,10 +90,10 @@ static void update_ime_text(vt_term_t *term, char *preedit_text, char *commit_te
       }
     }
 
-    if ((win = UIWINDOW_OF(term))) {
+    if (win) {
       ui_window_update(win, 3);
     }
-  } else /* if( commit_text) */
+  } else /* if (commit_text) */
   {
     (*utf8_parser->set_str)(utf8_parser, commit_text, strlen(commit_text));
     while (!utf8_parser->is_eos &&
@@ -141,10 +146,14 @@ static int need_resize(u_int cur_width,  /* contains scrollbar width and margin
 
         /* XXX */
         line_height = win->height / vt_term_get_rows(terms[count]);
+        if (line_height > 1 &&
+            win->height < ((vt_term_get_rows(terms[count]) + 1) * (line_height - 1))) {
+          line_height--;
+        }
 
         if (new_height <= win->y + line_height ||
             (vt_term_cursor_row(terms[count]) + 1) * line_height + win->y + win->vmargin <=
-                new_height) {
+            new_height) {
           return 0;
         }
       }
@@ -180,6 +189,7 @@ int ui_event_source_process(void) {
   u_int count;
   static u_int prev_num_terms;
   static int *fds;
+  static u_int32_t prev_msec;
 
   looper = ALooper_forThread();
 
@@ -213,6 +223,8 @@ int ui_event_source_process(void) {
   pthread_mutex_lock(&mutex);
 
   if (ident >= 0) {
+    struct timespec tm;
+
     if (!ui_display_process_event(source, ident)) {
       ALooper_removeFds(looper, fds, num_terms);
       prev_num_terms = 0;
@@ -248,8 +260,19 @@ int ui_event_source_process(void) {
          */
       }
     }
+
+    /* ALooper_pollAll() always returns in less than 100ms on some systems. */
+    if (clock_gettime(CLOCK_MONOTONIC, &tm) == 0) {
+      u_int32_t now_msec = tm.tv_sec * 1000 + tm.tv_nsec / 1000000;
+
+      if (now_msec < prev_msec || now_msec > prev_msec + 100) {
+        ui_display_idling(NULL);
+        prev_msec = now_msec;
+      }
+    }
   } else {
     ui_display_idling(NULL);
+    prev_msec += 100;
   }
 
   if (num_terms > 0) {
@@ -378,7 +401,7 @@ jboolean Java_mlterm_native_1activity_MLActivity_isSSH(JNIEnv *env, jobject this
 #ifdef USE_LIBSSH2
   vt_term_t *term;
   if ((term = get_current_term())) {
-    if (vt_term_get_slave_fd(term) == -1) { /* connecting to remote host. */
+    if (vt_term_get_pty_mode(term) == PTY_SSH) {
       return JNI_TRUE;
     }
   }
