@@ -8,6 +8,10 @@
 /* various AC_DEFINEs are defined in bl_config.h */
 #include "bl_config.h"
 
+#ifdef HAVE_STDINT_H
+#include <stdint.h> /* SIZE_MAX on some platforms */
+#endif
+
 #ifndef PATH_MAX
 #ifndef _POSIX_PATH_MAX
 #define _POSIX_PATH_MAX 255
