@@ -5,11 +5,32 @@
 #include <pobl/bl_util.h> /* BL_MIN */
 #include <pobl/bl_mem.h>  /* alloca */
 
+/*
+ * The size of the base palettes.
+ * SIXEL_PALETTE_SIZE should be less than 1024.
+ * (See the block of if ((color = params[0]) >= SIXEL_PALETTE_SIZE))
+ */
+#ifndef SIXEL_PALETTE_SIZE
+#define SIXEL_PALETTE_SIZE 256
+#endif
+
+/*
+ * Support sixel graphics extension which "sayaka --ormode on" outputs.
+ * (See https://github.com/isaki68k/sayaka/,
+ *  https://github.com/isaki68k/sayaka/blob/master/vala/sixel.native.c#L113)
+ */
+#if 0
+#define SIXEL_ORMODE
+#endif
+
 #ifdef SIXEL_1BPP
 
 #define correct_height correct_height_1bpp
 #define load_sixel_from_data load_sixel_from_data_1bpp
-#define SIXEL_RGB(r, g, b) ((9 * (r) + 30 * (g) + (b)) * 51 >= 5120 * 20 ? 1 : 0)
+#define SIXEL_RGB100(r, g, b) ((9 * (r) * (r) + 30 * (g) * (g) + (b) * (b) > \
+                                9 * 50 * 50 + 30 * 50 * 50 + 50 * 50) ? 1 : 0)
+#define SIXEL_RGB255(r, g, b) ((9 * (r) * (r) + 30 * (g) * (g) + (b) * (b) > \
+                                9 * 127 * 127 + 30 * 127 * 127 + 127 * 127) ? 1 : 0)
 #define CARD_HEAD_SIZE 0
 #define pixel_t u_int8_t
 
@@ -26,7 +47,7 @@
 
 #else
 
-#define SIXEL_RGB(r, g, b) ((((r)*255 / 100) << 16) | (((g)*255 / 100) << 8) | ((b)*255 / 100))
+#define SIXEL_RGB100(r, g, b) ((((r)*255 / 100) << 16) | (((g)*255 / 100) << 8) | ((b)*255 / 100))
 #ifndef CARD_HEAD_SIZE
 #ifdef GDK_PIXBUF_VERSION
 #define CARD_HEAD_SIZE 0
@@ -50,8 +71,8 @@ static pixel_t *custom_palette;
 
 #ifndef __GET_PARAMS__
 #define __GET_PARAMS__
-static size_t get_params(int *params, size_t max_params, const char **p) {
-  size_t count;
+static u_int get_params(int *params, u_int max_params, const char **p) {
+  u_int count;
   const char *start;
 
   memset(params, 0, sizeof(int) * max_params);
@@ -263,8 +284,8 @@ static void correct_height(pixel_t *pixels, int width, int *height /* multiple o
 static u_char *load_sixel_from_data(const char *file_data, u_int *width_ret, u_int *height_ret) {
   const char *p = file_data;
   u_char *pixels;
-  int params[5];
-  size_t n; /* number of params */
+  int params[6];
+  u_int n; /* number of params */
   int init_width;
   int pix_x;
   int pix_y;
@@ -276,6 +297,9 @@ static u_char *load_sixel_from_data(const char *file_data, u_int *width_ret, u_i
   int rep;
   int color;
   int asp_x;
+#ifdef SIXEL_ORMODE
+  int ormode = 0;
+#endif
 #ifdef SIXEL_SHAREPALETTE
   u_int8_t color_indexes[] =
     { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15,
@@ -297,24 +321,26 @@ static u_char *load_sixel_from_data(const char *file_data, u_int *width_ret, u_i
 #else
   /* VT340 Default Color Map */
   static pixel_t default_palette[] = {
-      SIXEL_RGB(0, 0, 0),    /* BLACK */
-      SIXEL_RGB(20, 20, 80), /* BLUE */
-      SIXEL_RGB(80, 13, 13), /* RED */
-      SIXEL_RGB(20, 80, 20), /* GREEN */
-      SIXEL_RGB(80, 20, 80), /* MAGENTA */
-      SIXEL_RGB(20, 80, 80), /* CYAN */
-      SIXEL_RGB(80, 80, 20), /* YELLOW */
-      SIXEL_RGB(53, 53, 53), /* GRAY 50% */
-      SIXEL_RGB(26, 26, 26), /* GRAY 25% */
-      SIXEL_RGB(33, 33, 60), /* BLUE* */
-      SIXEL_RGB(60, 26, 26), /* RED* */
-      SIXEL_RGB(33, 60, 33), /* GREEN* */
-      SIXEL_RGB(60, 33, 60), /* MAGENTA* */
-      SIXEL_RGB(33, 60, 60), /* CYAN* */
-      SIXEL_RGB(60, 60, 33), /* YELLOW* */
-      SIXEL_RGB(80, 80, 80)  /* GRAY 75% */
+      SIXEL_RGB100(0, 0, 0),    /* BLACK */
+      SIXEL_RGB100(20, 20, 80), /* BLUE */
+      SIXEL_RGB100(80, 13, 13), /* RED */
+      SIXEL_RGB100(20, 80, 20), /* GREEN */
+      SIXEL_RGB100(80, 20, 80), /* MAGENTA */
+      SIXEL_RGB100(20, 80, 80), /* CYAN */
+      SIXEL_RGB100(80, 80, 20), /* YELLOW */
+      SIXEL_RGB100(53, 53, 53), /* GRAY 50% */
+      SIXEL_RGB100(26, 26, 26), /* GRAY 25% */
+      SIXEL_RGB100(33, 33, 60), /* BLUE* */
+      SIXEL_RGB100(60, 26, 26), /* RED* */
+      SIXEL_RGB100(33, 60, 33), /* GREEN* */
+      SIXEL_RGB100(60, 33, 60), /* MAGENTA* */
+      SIXEL_RGB100(33, 60, 60), /* CYAN* */
+      SIXEL_RGB100(60, 60, 33), /* YELLOW* */
+      SIXEL_RGB100(80, 80, 80)  /* GRAY 75% */
   };
   pixel_t *palette;
+  pixel_t *base_palette;
+  pixel_t *ext_palette = NULL;
 #endif
 
   pixels = NULL;
@@ -324,18 +350,17 @@ static u_char *load_sixel_from_data(const char *file_data, u_int *width_ret, u_i
 #ifndef SIXEL_SHAREPALETTE
 #ifndef SIXEL_1BPP
   if (custom_palette) {
-    palette = custom_palette;
-    if (palette[256] == 0) /* No active palette */
-    {
+    base_palette = palette = custom_palette;
+    if (palette[SIXEL_PALETTE_SIZE] == 0) /* No active palette */ {
       memcpy(palette, default_palette, sizeof(default_palette));
-      memset(palette + 16, 0, sizeof(pixel_t) * 256 - sizeof(default_palette));
+      memset(palette + 16, 0, sizeof(pixel_t) * SIXEL_PALETTE_SIZE - sizeof(default_palette));
     }
   } else
 #endif
   {
-    palette = (pixel_t*)alloca(sizeof(pixel_t) * 256);
+    base_palette = palette = (pixel_t*)alloca(sizeof(pixel_t) * SIXEL_PALETTE_SIZE);
     memcpy(palette, default_palette, sizeof(default_palette));
-    memset(palette + 16, 0, sizeof(pixel_t) * 256 - sizeof(default_palette));
+    memset(palette + 16, 0, sizeof(pixel_t) * SIXEL_PALETTE_SIZE - sizeof(default_palette));
   }
 #endif
 
@@ -361,55 +386,55 @@ restart:
   if (*(++p) != ';') {
     /* P1 */
     switch (*p) {
-      case 'q':
-        /* The default value. (2:1 is documented though) */
-        asp_x = 1;
+    case 'q':
+      /* The default value. (2:1 is documented though) */
+      asp_x = 1;
 #if 0
-        asp_y = 1;
+      asp_y = 1;
 #endif
-        goto body;
+      goto body;
 
 #if 0
-      case '0':
-      case '1':
-      case '5':
-      case '6':
-        asp_x = 1;
-        asp_y = 2;
-        break;
+    case '0':
+    case '1':
+    case '5':
+    case '6':
+      asp_x = 1;
+      asp_y = 2;
+      break;
 
-      case '2':
-        asp_x = 1;
-        asp_y = 5;
-        break;
+    case '2':
+      asp_x = 1;
+      asp_y = 5;
+      break;
 
-      case '3':
-      case '4':
-        asp_x = 1;
-        asp_y = 3;
-        break;
+    case '3':
+    case '4':
+      asp_x = 1;
+      asp_y = 3;
+      break;
 
-      case '7':
-      case '8':
-      case '9':
-        asp_x = 1;
-        asp_y = 1;
-        break;
+    case '7':
+    case '8':
+    case '9':
+      asp_x = 1;
+      asp_y = 1;
+      break;
 
-      default:
+    default:
 #ifdef DEBUG
-        bl_debug_printf(BL_DEBUG_TAG " Illegal format.\n.");
+      bl_debug_printf(BL_DEBUG_TAG " Illegal format.\n.");
 #endif
-        goto end;
+      goto end;
 #else
-      case '\0':
+    case '\0':
 #ifdef DEBUG
-        bl_debug_printf(BL_DEBUG_TAG " Illegal format.\n.");
+      bl_debug_printf(BL_DEBUG_TAG " Illegal format.\n.");
 #endif
-        goto end;
+      goto end;
 
-      default:
-        asp_x = 1; /* XXX */
+    default:
+      asp_x = 1; /* XXX */
 #endif
     }
 
@@ -427,21 +452,27 @@ restart:
   if (*(++p) != ';') {
     /* P2 */
     switch (*p) {
-      case 'q':
-        goto body;
+    case 'q':
+      goto body;
+#ifdef SIXEL_ORMODE
+    case '5':
+      ormode = 1;
+      break;
+#endif
 #if 0
-      case '0':
-      case '2':
-        ... break;
+    case '0':
+    case '2':
+      ...
+      break;
 
-      default:
+    default:
 #else
-      case '\0':
+    case '\0':
 #endif
 #ifdef DEBUG
-        bl_debug_printf(BL_DEBUG_TAG " Illegal format.\n.");
+      bl_debug_printf(BL_DEBUG_TAG " Illegal format.\n.");
 #endif
-        goto end;
+      goto end;
     }
 
     if (p[1] == ';') {
@@ -526,47 +557,59 @@ body:
       a = 0x01;
       line = pixels + pix_y * stride + pix_x * PIXEL_SIZE;
 
+#ifdef SIXEL_ORMODE
+      if (ormode) {
+        for (y = 0; y < 6; y++) {
+          if ((b & a) != 0) {
+            int x;
+
+            for (x = 0; x < rep; x++) {
+              ((pixel_t*)line)[x] |= color;
+            }
+          }
+
+          a <<= 1;
+          line += stride;
+        }
+      } else
+#endif
+      {
 #ifdef SIXEL_SHAREPALETTE
-      for (y = 0; y < 6; y++) {
-        if ((b & a) != 0) {
+        for (y = 0; y < 6; y++) {
+          if ((b & a) != 0) {
 #ifdef USE_GRF
-          int x;
-          for (x = 0; x < rep; x++) {
-            ((u_int16_t*)line)[x] = color_indexes[color];
-          }
-#else
-          memset(line, color_indexes[color], rep);
-#endif
-        }
-        a <<= 1;
-        line += stride;
-      }
-#else
-      for (y = 0; y < 6; y++) {
-        if ((b & a) != 0) {
-          int x;
+            int x;
 
-          for (x = 0; x < rep; x++) {
-#if defined(GDK_PIXBUF_VERSION) || defined(USE_QUARTZ)
-            /* RGBA */
-            line[x * PIXEL_SIZE] = (palette[color] >> 16) & 0xff;
-            line[x * PIXEL_SIZE + 1] = (palette[color] >> 8) & 0xff;
-            line[x * PIXEL_SIZE + 2] = (palette[color]) & 0xff;
-            line[x * PIXEL_SIZE + 3] = 0xff;
-#elif defined(SIXEL_1BPP)
-            /* 0x80 is opaque mark */
-            ((pixel_t*)line)[x] = 0x80 | palette[color];
+            for (x = 0; x < rep; x++) {
+              ((u_int16_t*)line)[x] = color_indexes[color];
+            }
 #else
-            /* ARGB (cardinal) */
-            ((pixel_t*)line)[x] = 0xff000000 | palette[color];
+            memset(line, color_indexes[color], rep);
 #endif
           }
+          a <<= 1;
+          line += stride;
         }
+#else
+        for (y = 0; y < 6; y++) {
+          if ((b & a) != 0) {
+            int x;
 
-        a <<= 1;
-        line += stride;
-      }
+            for (x = 0; x < rep; x++) {
+#ifdef SIXEL_1BPP
+              /* 0x80 is opaque mark */
+              ((pixel_t*)line)[x] = 0x80 | palette[color];
+#else
+              ((pixel_t*)line)[x] = palette[color];
+#endif
+            }
+          }
+
+          a <<= 1;
+          line += stride;
+        }
 #endif /* SIXEL_SHAREPALETTE */
+      }
 
       pix_x += rep;
 
@@ -630,61 +673,55 @@ body:
         break;
       }
 
-      n = get_params(params, 5, &p);
+      n = get_params(params, sizeof(params) / sizeof(params[0]), &p);
 
       if (n > 0) {
-        if ((color = params[0]) < 0) {
-          color = 0;
-        } else if (color > 255) {
-          color = 255;
+        if ((color = params[0]) >= SIXEL_PALETTE_SIZE) {
+#ifdef SIXEL_SHAREPALETTE
+          color = SIXEL_PALETTE_SIZE - 1;
+#else
+          if (color >= 1024) {
+            color = 0;
+            goto use_base_palette;
+          } else {
+            color -= SIXEL_PALETTE_SIZE;
+
+            if (!ext_palette) {
+              if (!(ext_palette = (pixel_t*)alloca(sizeof(pixel_t) *
+                                                   (1024 - SIXEL_PALETTE_SIZE)))) {
+                color = 0;
+                goto use_base_palette;
+              }
+              memset(ext_palette, 0, 1024 - SIXEL_PALETTE_SIZE);
+            }
+
+            palette = ext_palette;
+          }
+#endif
+        } else {
+          if (color < 0) {
+            color = 0;
+          }
+
+#ifndef SIXEL_SHAREPALETTE
+        use_base_palette:
+          palette = base_palette;
+#endif
         }
       }
 
       if (n > 4) {
-        u_int8_t rgb[3];
+        int rgba[4]; /* The arguments of bl_hls_to_rgb() are 'int*' */
+
+        rgba[3] = 255;
 
         if (params[1] == 1) {
           /* HLS */
-          int h;
-          u_int32_t l;
-          u_int32_t s;
+          bl_hls_to_rgb(&rgba[0], &rgba[1], &rgba[2],
+                        BL_MIN(params[2], 360), BL_MIN(params[3], 100), BL_MIN(params[4], 100));
 
-          h = BL_MIN(params[2], 360);
-          l = BL_MIN(params[3], 100);
-          s = BL_MIN(params[4], 100);
-
-          if (s == 0) {
-            rgb[0] = rgb[1] = rgb[2] = l * 255 / 100;
-          } else {
-            u_int32_t m1;
-            u_int32_t m2;
-            int count;
-
-            if (l < 50) {
-              m2 = l * (100 + s);
-            } else {
-              m2 = (l + s) * 100 - l * s;
-            }
-            m1 = l * 200 - m2;
-
-            for (count = 0; count < 3; count++) {
-              u_int32_t pc;
-
-              if (h < 60) {
-                pc = m1 + (m2 - m1) * h / 60;
-              } else if (h < 180) {
-                pc = m2;
-              } else if (h < 240) {
-                pc = m1 + (m2 - m1) * (240 - h) / 60;
-              } else {
-                pc = m1;
-              }
-              rgb[count] = pc * 255 / 10000;
-
-              if ((h -= 120) < 0) {
-                h += 360;
-              }
-            }
+          if (n > 5 && params[5] < 100) {
+            rgba[3] = params[5] * 255 / 100;
           }
         } else if (params[1] == 2
 #ifndef SIXEL_SHAREPALETTE
@@ -692,27 +729,42 @@ body:
 #endif
                    ) {
           /* RGB */
-          rgb[0] = params[2] >= 100 ? 255 : params[2] * 255 / 100;
-          rgb[1] = params[3] >= 100 ? 255 : params[3] * 255 / 100;
-          rgb[2] = params[4] >= 100 ? 255 : params[4] * 255 / 100;
+          rgba[0] = params[2] >= 100 ? 255 : params[2] * 255 / 100;
+          rgba[1] = params[3] >= 100 ? 255 : params[3] * 255 / 100;
+          rgba[2] = params[4] >= 100 ? 255 : params[4] * 255 / 100;
+
+          if (n > 5 && params[5] < 100) {
+            rgba[3] = params[5] * 255 / 100;
+          }
+        } else if (params[1] == 3) {
+          /* R8G8B8 */
+          rgba[0] = params[2] >= 255 ? 255 : params[2];
+          rgba[1] = params[3] >= 255 ? 255 : params[3];
+          rgba[2] = params[4] >= 255 ? 255 : params[4];
+
+          if (n > 5 && params[5] < 255) {
+            rgba[3] = params[5];
+          }
         } else {
           continue;
         }
 
-#ifdef SIXEL_SHAREPALETTE
+#if defined(SIXEL_1BPP)
+        palette[color] = SIXEL_RGB255(rgba[0], rgba[1], rgba[2]);
+#elif defined(SIXEL_SHAREPALETTE)
         {
           u_int8_t r, g, b;
 
           /* fb/ui_display.h which defines ui_cmap_get_pixel_rgb() is included from ui_imagelib.c */
           if (!ui_cmap_get_pixel_rgb(&r, &g, &b, color) ||
-              abs(r - rgb[0]) >= 0x10 || abs(g - rgb[1]) >= 0x10 || abs(b - rgb[2]) >= 0x10) {
+              abs(r - rgba[0]) >= 0x10 || abs(g - rgba[1]) >= 0x10 || abs(b - rgba[2]) >= 0x10) {
             u_long closest;
 
             /*
              * fb/ui_display.h which defines ui_cmap_get_closest_color() is included
              * from ui_imagelib.c
              */
-            if (ui_cmap_get_closest_color(&closest, rgb[0], rgb[1], rgb[2])) {
+            if (ui_cmap_get_closest_color(&closest, rgba[0], rgba[1], rgba[2])) {
               color_indexes[color] = closest;
             } else {
               goto error;
@@ -720,16 +772,23 @@ body:
           }
         }
 #else
-        palette[color] = (rgb[0] << 16) | (rgb[1] << 8) | rgb[2];
+#if defined(GDK_PIXBUF_VERSION) || defined(USE_QUARTZ)
+        /* RGBA */
+        ((u_char*)(palette + color))[0] = rgba[0];
+        ((u_char*)(palette + color))[1] = rgba[1];
+        ((u_char*)(palette + color))[2] = rgba[2];
+        ((u_char*)(palette + color))[3] = rgba[3];
+#else
+        /* ARGB */
+        palette[color] = (rgba[3] << 24) | (rgba[0] << 16) | (rgba[1] << 8) | rgba[2];
 #endif
 
-#if !defined(SIXEL_1BPP) && !defined(SIXEL_SHAREPALETTE)
-        if (palette == custom_palette && palette[256] <= color) {
+        if (palette == custom_palette && palette[SIXEL_PALETTE_SIZE] <= color) {
           /*
            * Set max active palette number for NetBSD/OpenBSD.
            * (See load_file() in fb/ui_imagelib.c)
            */
-          palette[256] = color + 1;
+          palette[SIXEL_PALETTE_SIZE] = color + 1;
         }
 #endif
 
@@ -776,16 +835,22 @@ body:
       }
 
       if (width < new_width) {
-        need_resize = 1;
-        if (height > new_height) {
-          new_height = height;
+        /* w200 h100 -> w400 h0 case is ignored. */
+        if (new_height > 0) {
+          need_resize = 1;
+          if (height > new_height) {
+            new_height = height;
+          }
         }
       }
 
       if (height < new_height) {
-        need_resize = 1;
-        if (width > new_width) {
-          new_width = width;
+        /* w200 h100 -> w0 h200 case is ignored. */
+        if (new_width > 0) {
+          need_resize = 1;
+          if (width > new_width) {
+            new_width = width;
+          }
         }
       }
 
@@ -834,6 +899,24 @@ end:
     bl_debug_printf("Shrink size w %d h %d -> w %d h %d\n", width, height, cur_width, cur_height);
 #endif
 
+#ifdef SIXEL_ORMODE
+    if (ormode) {
+      int x, y;
+      pixel_t *p = pixels;
+
+      for (y = 0; y < cur_height; y++) {
+        for (x = 0; x < cur_width; x++) {
+#ifdef SIXEL_SHAREPALETTE
+          *p = color_indexes[*p];
+#else
+          *p = palette[*p];
+          p++;
+#endif
+        }
+      }
+    }
+#endif
+
     *width_ret = cur_width;
     *height_ret = cur_height;
 
@@ -847,7 +930,9 @@ end:
 #ifdef SIXEL_SHAREPALETTE
 error:
 #endif
-  free(pixels - CARD_HEAD_SIZE);
+  if (pixels) {
+    free(pixels - CARD_HEAD_SIZE);
+  }
 
   return NULL;
 }
@@ -871,7 +956,7 @@ static u_char *load_sixel_from_file(const char *path, u_int *width_ret, u_int *h
 pixel_t *ui_set_custom_sixel_palette(pixel_t *palette /* NULL -> Create new palette */
                                      ) {
   if (!palette) {
-    palette = (pixel_t*)calloc(sizeof(pixel_t), 257);
+    palette = (pixel_t*)calloc(sizeof(pixel_t), SIXEL_PALETTE_SIZE + 1);
   }
 
   return (custom_palette = palette);
@@ -882,7 +967,8 @@ pixel_t *ui_set_custom_sixel_palette(pixel_t *palette /* NULL -> Create new pale
 #undef realloc_pixels
 #undef correct_height
 #undef load_sixel_from_data
-#undef SIXEL_RGB
+#undef SIXEL_RGB100
+#undef SIXEL_RGB255
 #undef CARD_HEAD_SIZE
 #undef pixel_t
 #undef PIXEL_SIZE
