@@ -56,11 +56,16 @@ get_dev_num
 
 mlcc exec hsplit_screen 101
 mlcc exec next_screen
-mlcc exec vsplit_screen 30%
+mlcc exec vsplit_screen 50%
+mlcc exec hsplit_screen 50%
 
-set_config $dev_num "encoding=utf8"
 set_config `expr $dev_num - 1` "input_method=none"
+set_config $dev_num "encoding=eucjp"
 set_config `expr $dev_num + 1` "encoding=eucjp"
 
+mlcc exec next_screen
+mlcc exec next_screen
+mlcc exec hsplit_screen 50%
+
 emacsclient -t
 #$HOME/.sayaka/sayaka.sh s &
