@@ -51,7 +51,7 @@ static void end_application(GtkWidget *widget, gpointer data) { gtk_main_quit();
  */
 
 static int update(mc_io_t io) {
-  char *gui = mc_get_gui();
+  const char *gui = mc_get_gui();
 
   mc_update_char_encoding();
   mc_update_auto_detect();
@@ -460,7 +460,7 @@ static int show(void) {
   GtkWidget *label;
   GtkWidget *config_widget;
   GtkWidget *separator;
-  char *gui = mc_get_gui();
+  const char *gui = mc_get_gui();
 
   window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
   g_signal_connect(window, "destroy", G_CALLBACK(end_application), NULL);
@@ -799,6 +799,12 @@ int main(int argc, char **argv) {
   atexit(check_mem_leak);
 #endif
 
+  /*
+   * Loading libim-*.so after gtk im module is loaded might break internal states
+   * in gtk or input method library.
+   */
+  mc_im_init();
+
 #if !GTK_CHECK_VERSION(2, 90, 0)
   gtk_set_locale();
 #endif
