@@ -667,5 +667,7 @@ KeySym XStringToKeysym(char *str);
 #undef USE_REAL_VERTICAL_FONT
 #endif
 #undef NO_DISPLAY_FD
+#undef FLICK_SCROLL
+#undef UIWINDOW_SUPPORTS_PREEDITING
 
 #endif
