@@ -8,17 +8,22 @@
 #include <pobl/bl_privilege.h>
 #include <pobl/bl_unistd.h> /* bl_getuid/bl_getgid */
 #include <pobl/bl_mem.h>
-#include <pobl/bl_file.h> /* bl_file_set_cloexec */
 #include <pobl/bl_str.h>  /* bl_compare_str */
-#include <sys/select.h>
-#include <vt_term_manager.h>
-#include "../ui_screen_manager.h"
-#include "../ui_layout.h"
+#include "../ui_screen_manager.h" /* ui_mlclient */
+#include "../ui_event_source.h"
 #include "../ui_selection_encoding.h"
 
+/* XXX USE_CGLAYER is defined if 10.14 Mojave or later for now. */
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= 101400
+#define USE_CGLAYER
+#endif
+
 @interface MLTermView : NSView<NSTextInputClient> {
   ui_window_t *uiwindow;
   CGContextRef ctx;
+#ifdef USE_CGLAYER
+  CGLayerRef layer;
+#endif
   int forceExpose; /* 2 = visual bell */
 
   BOOL ignoreKeyDown;
@@ -65,14 +70,6 @@
 
 /* --- static variables --- */
 
-static struct {
-  int fd;
-  void (*handler)(void);
-
-} * additional_fds;
-
-static u_int num_additional_fds;
-
 static ui_window_t *uiwindow_for_mlterm_view;
 
 static NSMenuItem *configMenuItem;
@@ -116,83 +113,12 @@ static void monitor_pty(void) {
   bl_priv_change_egid(bl_getgid());
 #endif
 
-  dispatch_async(
-      dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
-        static int ret;
-        static fd_set read_fds;
-        static int maxfd;
-        static int is_cont_read;
-
-        for (;;) {
-          dispatch_sync(dispatch_get_main_queue(), ^{
-            vt_close_dead_terms();
-
-            vt_term_t **terms;
-            u_int num_terms = vt_get_all_terms(&terms);
-
-            int count;
-            int ptyfd;
-            if (ret > 0) {
-              for (count = 0; count < num_additional_fds; count++) {
-                if (additional_fds[count].fd < 0) {
-                  (*additional_fds[count].handler)();
-                }
-              }
-
-              for (count = 0; count < num_terms; count++) {
-                if ((ptyfd = vt_term_get_master_fd(terms[count])) >= 0 &&
-                    FD_ISSET(ptyfd, &read_fds)) {
-                  vt_term_parse_vt100_sequence(terms[count]);
-                }
-              }
-
-              if (++is_cont_read >= 2) {
-                [[NSRunLoop currentRunLoop]
-                    runUntilDate:[NSDate dateWithTimeIntervalSinceNow:0.01]];
-              }
-            } else if (ret == 0) {
-              ui_display_idling(NULL);
-              is_cont_read = 0;
-            }
-
-            FD_ZERO(&read_fds);
-            maxfd = -1;
-
-            for (count = 0; count < num_terms; count++) {
-              if ((ptyfd = vt_term_get_master_fd(terms[count])) >= 0) {
-                FD_SET(ptyfd, &read_fds);
-
-                if (ptyfd > maxfd) {
-                  maxfd = ptyfd;
-                }
-              }
-            }
-
-            for (count = 0; count < num_additional_fds; count++) {
-              if (additional_fds[count].fd >= 0) {
-                FD_SET(additional_fds[count].fd, &read_fds);
-
-                if (additional_fds[count].fd > maxfd) {
-                  maxfd = additional_fds[count].fd;
-                }
-              }
-            }
-          });
-
-          struct timeval tval;
-          tval.tv_usec = 100000; /* 0.1 sec */
-          tval.tv_sec = 0;
-
-          if (maxfd == -1 ||
-              (ret = select(maxfd + 1, &read_fds, NULL, NULL, &tval)) < 0) {
-            dispatch_sync(dispatch_get_main_queue(), ^{
-              exit_program();
-            });
-
-            break;
-          }
-        }
-      });
+  dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
+      ui_event_source_process();
+      dispatch_sync(dispatch_get_main_queue(), ^{
+          exit_program();
+        });
+    });
 }
 
 /* Undocumented */
@@ -285,58 +211,9 @@ static void drawUnistr(CGContextRef ctx, ui_font_t *font, unichar *str,
   }
 }
 
-static void (*orig_draw_preedit_str)(void *, vt_char_t *, u_int, int);
-
-static void dummy_draw_preedit_str(void *p, vt_char_t *chars, u_int num_chars,
-                                   int cursor_offset) {
-  /* Don't set ui_screen_t::is_preediting = 0. */
-}
-
 static void update_ime_text(ui_window_t *uiwindow, const char *preedit_text,
                             const char *cur_preedit_text) {
-  vt_term_t *term = ((ui_screen_t *)uiwindow)->term;
-
-  vt_term_set_config(term, "use_local_echo", "false");
-
-  if (orig_draw_preedit_str) {
-    ((ui_screen_t*)uiwindow)->im_listener.draw_preedit_str = orig_draw_preedit_str;
-    orig_draw_preedit_str = NULL;
-    ((ui_screen_t*)uiwindow)->is_preediting = 0;
-  }
-
-  ef_parser_t *utf8_parser;
-  if (!(utf8_parser = ui_get_selection_parser(1))) {
-    return;
-  }
-
-  (*utf8_parser->init)(utf8_parser);
-
-  if (*preedit_text == '\0') {
-    preedit_text = NULL;
-  } else {
-    if (bl_compare_str(preedit_text, cur_preedit_text) == 0) {
-      return;
-    }
-
-    /* Hide cursor */
-    orig_draw_preedit_str = ((ui_screen_t*)uiwindow)->im_listener.draw_preedit_str;
-    ((ui_screen_t*)uiwindow)->im_listener.draw_preedit_str = dummy_draw_preedit_str;
-    ((ui_screen_t*)uiwindow)->is_preediting = 1;
-
-    vt_term_parse_vt100_sequence(term);
-    vt_term_set_config(term, "use_local_echo", "true");
-
-    (*utf8_parser->set_str)(utf8_parser, preedit_text, strlen(preedit_text));
-
-    u_char buf[128];
-    size_t len;
-    while (!utf8_parser->is_eos &&
-           (len = vt_term_convert_to(term, buf, sizeof(buf), utf8_parser)) > 0) {
-      vt_term_preedit(term, buf, len);
-    }
-  }
-
-  ui_window_update(uiwindow, 3); /* UPDATE_SCREEN|UPDATE_CURSOR */
+  (*uiwindow->preedit)(uiwindow, preedit_text, cur_preedit_text);
 }
 
 static void remove_all_observers(ui_window_t *uiwindow) {
@@ -378,9 +255,9 @@ int cocoa_dialog_alert(const char *msg);
     uiwindow = uiwindow_for_mlterm_view;
   } else {
     if (global_args) {
-      char *args = bl_str_alloca_dup(global_args);
-      if (args) {
-        ui_mlclient(args, NULL);
+      char *args;
+      if ((args = alloca(strlen(global_args) + 1))) {
+        ui_mlclient(strcpy(args, global_args), NULL);
       }
     } else {
       char args[] = "mlclient";
@@ -456,6 +333,14 @@ int cocoa_dialog_alert(const char *msg);
 
 - (void)dealloc {
   [[NSNotificationCenter defaultCenter] removeObserver:self];
+
+#ifdef USE_CGLAYER
+  if (layer) {
+    CGLayerRelease(layer);
+    layer = nil;
+  }
+#endif
+
   [super dealloc];
 }
 
@@ -473,6 +358,17 @@ int cocoa_dialog_alert(const char *msg);
   }
 }
 
+#ifdef USE_CGLAYER
+- (void)setFrameSize:(NSSize)s {
+  if (layer) {
+    CGLayerRelease(layer);
+    layer = nil;
+  }
+
+  [super setFrameSize:s];
+}
+#endif
+
 static void reset_position(ui_window_t *uiwindow) {
   u_int count;
 
@@ -586,6 +482,9 @@ static void reset_position(ui_window_t *uiwindow) {
 
   if (!uiwindow->parent->my_window) {
     [[self window] useOptimizedDrawing:YES];
+#if 0
+    [[self window] allowsConcurrentViewDrawing:YES];
+#endif
 
     uiwindow->parent->my_window = [self window];
 
@@ -665,24 +564,50 @@ static void reset_position(ui_window_t *uiwindow) {
 
   XExposeEvent ev;
 
+#ifdef USE_CGLAYER
+  CGContextRef screen_ctx = [[NSGraphicsContext currentContext] graphicsPort];
+  CGContextSetBlendMode(screen_ctx, kCGBlendModeCopy);
+#else
   ctx = [[NSGraphicsContext currentContext] graphicsPort];
+#endif
+
 #if 0
   CGAffineTransform t = CGContextGetCTM(ctx);
   bl_debug_printf("%f %f %f %f %f %f\n", t.a, t.b, t.c, t.d, t.tx, t.ty);
 #endif
 
+#ifdef USE_CGLAYER
+  if (!layer) {
+    layer = CGLayerCreateWithContext(screen_ctx, self.bounds.size, NULL);
+    ctx = CGLayerGetContext(layer);
+
+    if (uiwindow->update_window_flag == 0) {
+      uiwindow->update_window_flag = 3; /* UPDATE_SCREEN|UPDATE_CURSOR (ui_screen.c) */
+    }
+    forceExpose = 1;
+  }
+
+  CGPoint p = CGPointMake(0, 0);
+#endif
+
+  CGContextSetBlendMode(ctx, kCGBlendModeCopy);
+
   if (forceExpose & 2) {
+    /* Visual bell */
     [self fillWith:&uiwindow->fg_color
                   :uiwindow->hmargin
                   :uiwindow->vmargin
                   :uiwindow->width
                   :uiwindow->height];
     CGContextFlush(ctx);
+#ifdef USE_CGLAYER
+    CGContextDrawLayerAtPoint(screen_ctx, p, layer);
+#endif
 
     [[NSRunLoop currentRunLoop]
         runUntilDate:[NSDate dateWithTimeIntervalSinceNow:0.1]];
 
-    forceExpose = 1;
+    forceExpose &= ~2;
     uiwindow->update_window_flag = 0;
   }
 
@@ -697,6 +622,10 @@ static void reset_position(ui_window_t *uiwindow) {
 
   forceExpose = 0;
   uiwindow->update_window_flag = 0;
+
+#ifdef USE_CGLAYER
+  CGContextDrawLayerAtPoint(screen_ctx, p, layer);
+#endif
 }
 
 - (BOOL)isOpaque {
@@ -763,6 +692,10 @@ static ui_window_t *get_current_window(ui_window_t *win) {
 }
 
 - (BOOL)becomeFirstResponder {
+  if ([super becomeFirstResponder] != YES) {
+    return NO;
+  }
+
   XEvent ev;
 
   ev.type = UI_KEY_FOCUS_IN;
@@ -1036,14 +969,14 @@ static ui_window_t *get_current_window(ui_window_t *win) {
 
 - (NSDragOperation)draggingEntered:(id<NSDraggingInfo>)sender {
   NSPasteboard *pboard = [sender draggingPasteboard];
-  NSDragOperation sourceMask = [sender draggingSourceOperationMask];
 
   if ([[pboard types] containsObject:NSFilenamesPboardType]) {
-    if (sourceMask & NSDragOperationLink) {
-      return NSDragOperationLink;
-    } else if (sourceMask & NSDragOperationCopy) {
-      return NSDragOperationCopy;
-    }
+#ifdef __DEBUG
+    NSDragOperation sourceMask = [sender draggingSourceOperationMask];
+    bl_debug_printf("draggingEntered SourceMask %d\n", sourceMask);
+#endif
+
+    return NSDragOperationGeneric;
   }
 
   return NSDragOperationNone;
@@ -1053,16 +986,28 @@ static ui_window_t *get_current_window(ui_window_t *win) {
   NSPasteboard *pboard = [sender draggingPasteboard];
 
   if ([[pboard types] containsObject:NSFilenamesPboardType]) {
+    NSDragOperation sourceMask = [sender draggingSourceOperationMask];
     NSArray *files = [pboard propertyListForType:NSFilenamesPboardType];
     int count;
-    XSelectionNotifyEvent ev;
 
-    ev.type = UI_SELECTION_NOTIFIED;
-    for (count = 0; count < [files count]; count++) {
-      ev.data = [[files objectAtIndex:count] UTF8String];
-      ev.len = strlen(ev.data);
+#ifdef __DEBUG
+    bl_debug_printf("performDragOperation SourceMask %d\n", sourceMask);
+#endif
 
-      ui_window_receive_event(uiwindow, (XEvent *)&ev);
+    if (sourceMask == NSDragOperationGeneric /* Command */ && uiwindow->set_xdnd_config) {
+      for (count = 0; count < [files count]; count++) {
+        (*uiwindow->set_xdnd_config)(uiwindow, NULL, "scp",
+                                     [[files objectAtIndex:count] UTF8String]);
+      }
+    } else {
+      XSelectionNotifyEvent ev;
+
+      ev.type = UI_SELECTION_NOTIFIED;
+      for (count = 0; count < [files count]; count++) {
+        ev.data = [[files objectAtIndex:count] UTF8String];
+        ev.len = strlen(ev.data);
+        ui_window_receive_event(uiwindow, (XEvent *)&ev);
+      }
     }
   }
 
@@ -1227,7 +1172,7 @@ static ui_window_t *get_current_window(ui_window_t *win) {
 #endif
 
 #if 0
-  CGContextSelectFont(ctx, "Menlo", 1, kCGEncodingMacRoman);
+  CGContextSelectFont(ctx, "Menlo", 16, kCGEncodingMacRoman);
   CGContextShowTextAtPoint(ctx, x, ACTUAL_HEIGHT(uiwindow) - y - 1, str, len);
 #else
   unichar ustr[len];
@@ -1257,23 +1202,21 @@ static ui_window_t *get_current_window(ui_window_t *win) {
   color->pixel += (0x10 * (count++));
 #endif
 
+  CGFloat alpha;
   if (IS_OPAQUE) {
-    set_fill_color(color);
-
-    CGRect rect =
-        CGRectMake(x, ACTUAL_HEIGHT(uiwindow) - y - height, width, height);
-    CGContextAddRect(ctx, rect);
-    CGContextFillPath(ctx);
+    alpha = 1.0;
   } else {
-    [[NSColor colorWithDeviceRed:((color->pixel >> 16) & 0xff) / 255.0
-                           green:((color->pixel >> 8) & 0xff) / 255.0
-                            blue:(color->pixel & 0xff) / 255.0
-                           alpha:((color->pixel >> 24) & 0xff) / 255.0] set];
-
-    NSRectFillUsingOperation(
-        NSMakeRect(x, ACTUAL_HEIGHT(uiwindow) - y - height, width, height),
-        NSCompositeCopy);
+    alpha = (((color)->pixel >> 24) & 0xff) / 255.0;
   }
+
+  CGContextSetRGBFillColor(ctx, (((color)->pixel >> 16) & 0xff) / 255.0,
+                           (((color)->pixel >> 8) & 0xff) / 255.0,
+                           ((color)->pixel & 0xff) / 255.0, alpha);
+
+  CGRect rect =
+    CGRectMake(x, ACTUAL_HEIGHT(uiwindow) - y - height, width, height);
+  CGContextAddRect(ctx, rect);
+  CGContextFillPath(ctx);
 }
 
 - (void)drawRectFrame:(ui_color_t *)color:(int)x1:(int)y1:(int)x2:(int)y2 {
@@ -1312,20 +1255,20 @@ static ui_window_t *get_current_window(ui_window_t *win) {
 
   /* Don't release this CGImage */
   NSRect src_r = NSMakeRect(src_x, ACTUAL_HEIGHT(uiwindow) - src_y - height,
-			width, height);
+      width, height);
   NSBitmapImageRep *bir = [self bitmapImageRepForCachingDisplayInRect:src_r];
   [self cacheDisplayInRect:src_r toBitmapImageRep:bir];
   CGImageRef image = bir.CGImage;
 
   CGRect dst_r = CGRectMake(dst_x, ACTUAL_HEIGHT(uiwindow) - dst_y - height,
-				width, height);
+        width, height);
   CGContextDrawImage(ctx, dst_r, image);
 
   static int i;
   if (i == 0) {
     CFURLRef url = [NSURL fileURLWithPath:@"/Users/ken/kallist/log.png"];
     CGImageDestinationRef dest = CGImageDestinationCreateWithURL(
-				   url, kUTTypePNG, 1, NULL);
+           url, kUTTypePNG, 1, NULL);
     CGImageDestinationAddImage(dest, image, nil);
     CGImageDestinationFinalize(dest);
     CFRelease( dest);
@@ -1346,6 +1289,10 @@ static ui_window_t *get_current_window(ui_window_t *win) {
   forceExpose |= flag;
 
   if (IS_OPAQUE || forceExpose) {
+    /*
+     * setNeedsDisplay:YES calls drawRect with full screen 'rect'.
+     * But if forceExpose is 0, expose() in ui_window.c ignores it.
+     */
     [self setNeedsDisplay:YES];
   } else {
     int x;
@@ -1599,13 +1546,11 @@ u_int cocoa_font_get_advance(CGFontRef cg_font, u_int fontsize, int size_attr,
   return advance * fontsize / CGFontGetUnitsPerEm(cg_font);
 }
 
-int cocoa_clipboard_own(MLTermView *view) {
+void cocoa_clipboard_own(MLTermView *view) {
   NSPasteboard *pasteboard = [NSPasteboard generalPasteboard];
 
   [pasteboard declareTypes:[NSArray arrayWithObject:NSPasteboardTypeString]
                      owner:view];
-
-  return 1;
 }
 
 void cocoa_clipboard_set(const u_char *utf8, size_t len) {
@@ -1655,54 +1600,6 @@ CGImageRef cocoa_load_image(const char *path, u_int *width, u_int *height) {
   return cgimg;
 }
 
-/*
- * fd >= 0  -> Normal file descriptor. handler is invoked if fd is ready.
- * fd < 0 -> Special ID. handler is invoked at interval of 0.1 sec.
- */
-int cocoa_add_fd(int fd, void (*handler)(void)) {
-  void *p;
-
-  if (!handler) {
-    return 0;
-  }
-
-  if ((p = realloc(additional_fds, sizeof(*additional_fds) *
-                                       (num_additional_fds + 1))) == NULL) {
-    return 0;
-  }
-
-  additional_fds = p;
-  additional_fds[num_additional_fds].fd = fd;
-  additional_fds[num_additional_fds++].handler = handler;
-  if (fd >= 0) {
-    bl_file_set_cloexec(fd);
-  }
-
-#ifdef DEBUG
-  bl_debug_printf(BL_DEBUG_TAG " %d is added to additional fds.\n", fd);
-#endif
-
-  return 1;
-}
-
-int cocoa_remove_fd(int fd) {
-  u_int count;
-
-  for (count = 0; count < num_additional_fds; count++) {
-    if (additional_fds[count].fd == fd) {
-#ifdef DEBUG
-      bl_debug_printf(BL_DEBUG_TAG " Additional fd %d is removed.\n", fd);
-#endif
-
-      additional_fds[count] = additional_fds[--num_additional_fds];
-
-      return 1;
-    }
-  }
-
-  return 0;
-}
-
 const char *cocoa_get_bundle_path(void) {
   return [[[NSBundle mainBundle] bundlePath] UTF8String];
 }
@@ -1727,7 +1624,7 @@ char *cocoa_dialog_password(const char *msg) {
 int cocoa_dialog_okcancel(const char *msg) {
   NSAlert *alert = create_dialog(msg, 1);
   if (alert == nil) {
-    return NULL;
+    return 0;
   }
 
   if ([alert runModal] == NSAlertFirstButtonReturn) {
