@@ -25,8 +25,8 @@
 
 #define ParentRelative (1L)
 
-#define IS_XSCREEN(win) ((win)->selection_cleared)
-#define IS_XLAYOUT(win) ((win)->child_window_resized)
+#define IS_UISCREEN(win) ((win)->selection_cleared)
+#define IS_UILAYOUT(win) ((win)->child_window_resized)
 
 #if 0
 #define DEBUG_SCROLLABLE
@@ -38,7 +38,6 @@
 
 /* --- static variables --- */
 
-static int click_interval = 250; /* millisecond, same as xterm. */
 static int use_urgent_bell;
 
 /* --- static functions --- */
@@ -100,46 +99,15 @@ static void notify_move_to_children(ui_window_t *win) {
   }
 }
 
-static u_int total_min_width(ui_window_t *win) {
-  int count;
-  u_int min_width;
-
-  min_width = win->min_width + win->hmargin * 2;
-
-  for (count = 0; count < win->num_children; count++) {
-    if (win->children[count]->is_mapped) {
-      /* XXX */
-      min_width += total_min_width(win->children[count]);
-    }
-  }
-
-  return min_width;
-}
-
-static u_int total_min_height(ui_window_t *win) {
-  int count;
-  u_int min_height;
-
-  min_height = win->min_height + win->vmargin * 2;
-
-  for (count = 0; count < win->num_children; count++) {
-    if (win->children[count]->is_mapped) {
-      /* XXX */
-      min_height += total_min_height(win->children[count]);
-    }
-  }
-
-  return min_height;
-}
-
 static u_int total_width_inc(ui_window_t *win) {
-  int count;
+  u_int count;
   u_int width_inc;
 
   width_inc = win->width_inc;
 
   for (count = 0; count < win->num_children; count++) {
-    if (win->children[count]->is_mapped) {
+    if (win->children[count]->is_mapped &&
+        (win->children[count]->sizehint_flag & SIZEHINT_WIDTH)) {
       u_int sub_inc;
 
       /*
@@ -156,13 +124,14 @@ static u_int total_width_inc(ui_window_t *win) {
 }
 
 static u_int total_height_inc(ui_window_t *win) {
-  int count;
+  u_int count;
   u_int height_inc;
 
   height_inc = win->height_inc;
 
   for (count = 0; count < win->num_children; count++) {
-    if (win->children[count]->is_mapped) {
+    if (win->children[count]->is_mapped &&
+        (win->children[count]->sizehint_flag & SIZEHINT_HEIGHT)) {
       u_int sub_inc;
 
       /*
@@ -337,6 +306,7 @@ int ui_window_init(ui_window_t *win, u_int width, u_int height, u_int min_width,
   win->min_height = min_height;
   win->width_inc = width_inc;
   win->height_inc = height_inc;
+  win->sizehint_flag = SIZEHINT_WIDTH|SIZEHINT_HEIGHT;
   win->hmargin = hmargin;
   win->vmargin = vmargin;
 
@@ -373,9 +343,9 @@ void ui_window_final(ui_window_t *win) {
     /*
      * win->parent may be NULL because this function is called
      * from ui_layout after ui_window_remove_child(),
-     * not only win->parent but also IS_XLAYOUT(win) is necessary.
+     * not only win->parent but also IS_UILAYOUT(win) is necessary.
      */
-    if (!win->parent && IS_XLAYOUT(win)) {
+    if (!win->parent && IS_UILAYOUT(win)) {
       window_dealloc(my_window);
     } else {
       view_dealloc(my_window);
@@ -475,7 +445,7 @@ int ui_window_set_bg_color(ui_window_t *win, ui_color_t *bg_color) {
 
   win->bg_color = *bg_color;
 
-  if (win->my_window && IS_XSCREEN(win)) {
+  if (win->my_window && IS_UISCREEN(win)) {
     view_bg_color_changed(win->my_window);
     ui_window_update_all(win);
   }
@@ -599,7 +569,7 @@ int ui_window_show(ui_window_t *win,
     ui_window_set_transparent(win, win->pic_mod);
   }
 
-  if (IS_XSCREEN(win) && win->parent->my_window) {
+  if (IS_UISCREEN(win) && win->parent->my_window) {
     view_alloc(win);
   }
 
@@ -641,6 +611,14 @@ int ui_window_resize(ui_window_t *win, u_int width, /* excluding margin */
     return 0;
   }
 
+#ifdef COCOA_TOUCH
+  if (!win->parent) {
+    /* Don't allow to change the size of a root window. */
+    flag |= NOTIFY_TO_MYSELF;
+    goto end;
+  }
+#endif
+
   /* Max width of each window is screen width. */
   if ((flag & LIMIT_RESIZE) && win->disp->width < width) {
     win->width = win->disp->width - win->hmargin * 2;
@@ -648,7 +626,7 @@ int ui_window_resize(ui_window_t *win, u_int width, /* excluding margin */
     win->width = width;
   }
 
-  /* Maui.height of each window is screen height. */
+  /* Max.height of each window is screen height. */
   if ((flag & LIMIT_RESIZE) && win->disp->height < height) {
     win->height = win->disp->height - win->vmargin * 2;
   } else {
@@ -656,8 +634,13 @@ int ui_window_resize(ui_window_t *win, u_int width, /* excluding margin */
   }
 
   if (win->parent) {
+#ifdef COCOA_TOUCH
+    view_set_rect(win->my_window, win->x, win->y,
+                  ACTUAL_WIDTH(win), ACTUAL_HEIGHT(win));
+#else
     view_set_rect(win->my_window, win->x, ACTUAL_HEIGHT(win->parent) - ACTUAL_HEIGHT(win) - win->y,
                   ACTUAL_WIDTH(win), ACTUAL_HEIGHT(win));
+#endif
 
     if ((flag & NOTIFY_TO_PARENT) && win->parent->child_window_resized) {
       (*win->parent->child_window_resized)(win->parent, win);
@@ -666,13 +649,16 @@ int ui_window_resize(ui_window_t *win, u_int width, /* excluding margin */
     /*
      * win->parent may be NULL because this function is called
      * from ui_layout after ui_window_remove_child(),
-     * not only win->parent but also IS_XLAYOUT(win) is necessary.
+     * not only win->parent but also IS_UILAYOUT(win) is necessary.
      */
-    if (IS_XLAYOUT(win)) {
+    if (IS_UILAYOUT(win)) {
       window_resize(win->my_window, ACTUAL_WIDTH(win), ACTUAL_HEIGHT(win));
     }
   }
 
+#ifdef COCOA_TOUCH
+end:
+#endif
   if ((flag & NOTIFY_TO_MYSELF) && win->window_resized) {
     (*win->window_resized)(win);
     win->update_window_flag = 0; /* force redraw */
@@ -742,8 +728,6 @@ void ui_window_set_normal_hints(ui_window_t *win, u_int min_width, u_int min_hei
   root = ui_get_root_window(win);
 
   window_set_normal_hints(root->my_window, total_width_inc(root), total_height_inc(root));
-
-  return 1;
 }
 
 void ui_window_set_override_redirect(ui_window_t *win, int flag) {}
@@ -764,8 +748,13 @@ int ui_window_move(ui_window_t *win, int x, int y) {
   win->y = y;
 
   if (win->parent) {
+#ifdef COCOA_TOUCH
+    view_set_rect(win->my_window, x, y,
+                  ACTUAL_WIDTH(win), ACTUAL_HEIGHT(win));
+#else
     view_set_rect(win->my_window, x, ACTUAL_HEIGHT(win->parent) - ACTUAL_HEIGHT(win) - y,
                   ACTUAL_WIDTH(win), ACTUAL_HEIGHT(win));
+#endif
   }
 
   return 1;
@@ -869,7 +858,7 @@ void ui_window_update(ui_window_t *win, int flag) {
 void ui_window_update_all(ui_window_t *win) {
   u_int count;
 
-  if (IS_XSCREEN(win)) {
+  if (IS_UISCREEN(win)) {
     view_update(win->my_window, 1);
   }
 
@@ -1005,8 +994,8 @@ int ui_window_receive_event(ui_window_t *win, XEvent *event) {
     case UI_CLOSE_WINDOW:
       /* root window */
       win->my_window = None;
-      if (win->window_deleted) {
-        (*win->window_deleted)(win);
+      if (win->window_destroyed) {
+        (*win->window_destroyed)(win);
       }
       break;
   }
@@ -1182,6 +1171,11 @@ void ui_set_use_clipboard_selection(int use_it) {}
 int ui_is_using_clipboard_selection(void) { return 0; }
 
 int ui_window_set_selection_owner(ui_window_t *win, Time time) {
+#ifdef COCOA_TOUCH
+  if (win->utf_selection_requested) {
+    (*win->utf_selection_requested)(win, NULL, time);
+  }
+#else
   if (ui_window_is_selection_owner(win)) {
     /* Already owner */
 
@@ -1193,6 +1187,7 @@ int ui_window_set_selection_owner(ui_window_t *win, Time time) {
 #endif
 
   cocoa_clipboard_own(win->my_window);
+#endif
 
   return 1;
 }
@@ -1239,13 +1234,9 @@ void ui_window_remove_icon(ui_window_t *win) {}
 
 void ui_window_reset_group(ui_window_t *win) {}
 
-void ui_set_click_interval(int interval) {
-  click_interval = interval;
-}
+void ui_set_click_interval(int interval) {}
 
-int ui_get_click_interval(void) {
-  return click_interval;
-}
+int ui_get_click_interval(void) { return 250; /* dummy */ }
 
 u_int ui_window_get_mod_ignore_mask(ui_window_t *win, KeySym *keysyms) { return ~0; }
 
