@@ -12,12 +12,10 @@
 #include <pobl/bl_util.h> /* BL_SWAP */
 #include <pobl/bl_dialog.h>
 
-#include <mef/ef_utf8_parser.h>
-
 #include "../ui_window.h"
 #include "../ui_picture.h"
 #include "../ui_imagelib.h"
-#include "../ui_screen.h" /* update_ime_text */
+#include "../ui_selection_encoding.h"
 #include "syswminfo.h"
 
 #ifndef USE_WIN32API
@@ -46,7 +44,6 @@ static int rotate_display;
 static Uint32 pty_event_type;
 static Uint32 vsync_interval_msec;
 static Uint32 next_vsync_msec;
-static ef_parser_t *utf8_parser;
 static u_char *cur_preedit_text;
 static SDL_threadID main_tid;
 #ifdef MONITOR_PTY
@@ -142,43 +139,19 @@ static ui_window_t *search_inputtable_window(ui_window_t *candidate, ui_window_t
 }
 
 static void update_ime_text(ui_window_t *uiwindow, const char *preedit_text) {
-  vt_term_t *term;
-
   if (!(uiwindow = search_focused_window(uiwindow)) || !uiwindow->inputtable) {
     return;
   }
-  term = ((ui_screen_t*)uiwindow)->term;
 
-  if (cur_preedit_text) {
-    vt_term_set_config(term, "use_local_echo", "false");
-  }
-
-  (*utf8_parser->init)(utf8_parser);
+  (*uiwindow->preedit)(uiwindow, preedit_text, cur_preedit_text);
 
   if (*preedit_text == '\0') {
     free(cur_preedit_text);
     cur_preedit_text = NULL;
   } else {
-    u_char buf[128];
-    size_t len;
-
-    if (bl_compare_str(preedit_text, cur_preedit_text) == 0) {
-      return;
-    }
-
-    vt_term_set_config(term, "use_local_echo", "true");
-
-    (*utf8_parser->set_str)(utf8_parser, preedit_text, strlen(preedit_text));
-    while (!utf8_parser->is_eos &&
-           (len = vt_term_convert_to(term, buf, sizeof(buf), utf8_parser)) > 0) {
-      vt_term_preedit(term, buf, len);
-    }
-
     free(cur_preedit_text);
     cur_preedit_text = strdup(preedit_text);
   }
-
-  ui_window_update(uiwindow, 3); /* UPDATE_SCREEN|UPDATE_CURSOR */
 }
 
 /*
@@ -284,10 +257,12 @@ static u_int total_height_inc(ui_window_t *win) {
   return height_inc;
 }
 
-static int check_resize(u_int old_width, u_int old_height, int32_t *new_width, int32_t *new_height,
-                        u_int min_width, u_int min_height, u_int width_inc, u_int height_inc,
-                        int check_inc) {
+static int modify_resize(u_int old_width, u_int old_height, int32_t *new_width, int32_t *new_height,
+                         u_int min_width, u_int min_height, u_int width_inc, u_int height_inc,
+                         int check_inc) {
   u_int diff;
+  u_int orig_new_width = *new_width;
+  u_int orig_new_height = *new_height;
 
   if (rotate_display) {
     BL_SWAP(unsigned int, min_width, min_height);
@@ -330,7 +305,7 @@ static int check_resize(u_int old_width, u_int old_height, int32_t *new_width, i
     }
   }
 
-  if (old_width == *new_width && old_height == *new_height) {
+  if (orig_new_width == *new_width && orig_new_height == *new_height) {
     return 0;
   } else {
     return 1;
@@ -611,6 +586,9 @@ static u_int get_mod_state(SDL_Keymod mod) {
   if (mod & KMOD_ALT) {
     state |= Mod1Mask;
   }
+  if (mod & KMOD_GUI) {
+    state |= CommandMask;
+  }
 
   return state;
 }
@@ -681,8 +659,8 @@ static void poll_event(void) {
       disp = get_display(ev.window.windowID);
 
       for (count = 0; count < disp->num_roots; count++) {
-        if (disp->roots[count]->window_deleted) {
-          (*disp->roots[count]->window_deleted)(disp->roots[count]);
+        if (disp->roots[count]->window_destroyed) {
+          (*disp->roots[count]->window_destroyed)(disp->roots[count]);
         }
       }
     }
@@ -699,7 +677,8 @@ static void poll_event(void) {
     xev.xkey.state = get_mod_state(ev.key.keysym.mod);
 
     if (!cur_preedit_text &&
-        (xev.xkey.ksym < 0x20 || xev.xkey.ksym >= 0x7f || xev.xkey.state == ControlMask)) {
+        (xev.xkey.ksym < 0x20 || xev.xkey.ksym >= 0x7f || xev.xkey.state == ControlMask ||
+         xev.xkey.state == CommandMask)) {
       ui_window_receive_event(get_display(ev.key.windowID)->roots[0], &xev);
     }
 
@@ -721,7 +700,7 @@ static void poll_event(void) {
         xev.xkey.keycode = 0;
       }
       xev.xkey.str = ev.text.text;
-      xev.xkey.parser = utf8_parser;
+      xev.xkey.parser = ui_get_selection_parser(1);
       xev.xkey.state = get_mod_state(SDL_GetModState());
 
       ui_window_receive_event(win, &xev);
@@ -767,6 +746,25 @@ static void poll_event(void) {
  
     break;
 
+  case SDL_MOUSEWHEEL:
+    if (ev.wheel.y) {
+      disp = get_display(ev.window.windowID);
+
+      xev.xbutton.type = ButtonPress;
+      xev.xbutton.time = ev.button.timestamp;
+      SDL_GetMouseState(&xev.xbutton.x, &xev.xbutton.y);
+      xev.xbutton.state = get_mod_state(SDL_GetModState());
+
+      if (ev.wheel.y > 0) {
+        xev.xbutton.button = 4;
+      } else {
+        xev.xbutton.button = 5;
+      }
+
+      receive_mouse_event(disp, &xev.xbutton);
+    }
+    break;
+
   case SDL_MOUSEMOTION:
     disp = get_display(ev.window.windowID);
 
@@ -788,6 +786,11 @@ static void poll_event(void) {
 
     receive_mouse_event(disp, &xev.xmotion);
 
+#if 0 /* defined(__HAIKU__) */
+    /* If mouse cursor moves, garbage is left on HaikuOS, so damaged = 1 to redraw screen. */
+    disp->display->damaged = 1;
+#endif
+
     break;
 
   case SDL_WINDOWEVENT:
@@ -798,9 +801,9 @@ static void poll_event(void) {
       u_int height = ev.window.data2;
 
 #if 0
-      if (check_resize(disp->display->width, disp->display->height, &width, &height,
-                       total_min_width(disp->roots[0]), total_min_height(disp->roots[0]),
-                       total_width_inc(disp->roots[0]), total_height_inc(disp->roots[0]), 1)) {
+      if (modify_resize(disp->display->width, disp->display->height, &width, &height,
+                        total_min_width(disp->roots[0]), total_min_height(disp->roots[0]),
+                        total_width_inc(disp->roots[0]), total_height_inc(disp->roots[0]), 1)) {
         SDL_SetWindowSize(disp->display->window, width, height);
       } else
 #endif
@@ -818,6 +821,11 @@ static void poll_event(void) {
         init_display(disp->display, NULL, 0, 0, 0);
         disp->display->resizing = 0;
         ui_window_resize_with_margin(disp->roots[0], disp->width, disp->height, NOTIFY_TO_MYSELF);
+
+#if 1
+        /* This is because ui_window_resize_with_margin() redraws screen partially. */
+        ui_window_update_all(disp->roots[0]);
+#endif
       }
     } else if (ev.window.event == SDL_WINDOWEVENT_FOCUS_GAINED) {
       ui_window_t *win;
@@ -832,9 +840,27 @@ static void poll_event(void) {
         xev.type = FocusOut;
         ui_window_receive_event(win, &xev);
       }
+    } else if (ev.window.event == SDL_WINDOWEVENT_EXPOSED) {
+      get_display(ev.window.windowID)->display->damaged = 1;
     }
+
     break;
 
+  case SDL_DROPFILE:
+    {
+      ui_window_t *win = get_display(ev.key.windowID)->roots[0];
+
+      if ((SDL_GetModState() & KMOD_SHIFT) && win->set_xdnd_config) {
+        (*win->set_xdnd_config)(win, NULL, "scp", ev.drop.file);
+      } else if (win->utf_selection_notified) {
+        (*win->utf_selection_notified)(win, ev.drop.file, strlen(ev.drop.file));
+      }
+
+      SDL_free(ev.drop.file);
+
+      break;
+    }
+
   default:
     if (ev.type == pty_event_type) {
 #ifdef MONITOR_PTY
@@ -853,10 +879,6 @@ ui_display_t *ui_display_open(char *disp_name, u_int depth) {
   void *p;
   struct rgb_info rgbinfo = {0, 0, 0, 16, 8, 0};
 
-  if (!utf8_parser && !(utf8_parser = ef_utf8_parser_new())) {
-    return NULL;
-  }
-
   if (!(disp = calloc(1, sizeof(ui_display_t) + sizeof(Display)))) {
     return NULL;
   }
@@ -886,13 +908,16 @@ ui_display_t *ui_display_open(char *disp_name, u_int depth) {
 
     main_tid = SDL_GetThreadID(NULL);
 
+    SDL_EventState(SDL_DROPFILE, SDL_ENABLE);
+
     num_displays = 1;
 
 #ifdef MONITOR_PTY
     {
-      pty_cond = SDL_CreateCond();
+      SDL_Thread *thrd;
 
-      SDL_Thread *thrd = SDL_CreateThread(monitor_ptys, "pty_thread", NULL);
+      pty_cond = SDL_CreateCond();
+      thrd = SDL_CreateThread(monitor_ptys, "pty_thread", NULL);
       SDL_DetachThread(thrd);
     }
 #endif
@@ -923,9 +948,6 @@ void ui_display_close(ui_display_t *disp) {
         free(cur_preedit_text);
         cur_preedit_text = NULL;
 
-        (*utf8_parser->delete)(utf8_parser);
-        utf8_parser = NULL;
-
 #ifdef MONITOR_PTY
         SDL_CondSignal(pty_cond);
         SDL_DestroyCond(pty_cond);
