@@ -265,22 +265,17 @@ check_style:
   return font;
 }
 
-static int parse_value(char **font_name, /* if value is "" or illegal format, not changed. */
-                       char *value       /* Don't specify NULL. */
-                       ) {
-  if (strchr(value, ',')) {
-    /* XXX Compat with old format (3.6.3 or before): [size],[font name] */
+static void parse_value(char **font_name, /* if value is "" or illegal format, not changed. */
+                        char *value       /* Don't specify NULL. */ ) {
+#if 1 /* XXX Compat with old format (3.6.3 or before): [size],[font name] */
+  char *p;
 
-    /*
-     * bl_str_sep() never returns NULL and and value never becomes NULL
-     * because strchr( value , ',') is succeeded.
-     */
-    bl_str_sep(&value, ",");
+  if ('0' <= *value && *value <= '9' && (p = strchr(value, ','))) {
+    value = p + 1;
   }
+#endif
 
   *font_name = value;
-
-  return 1;
 }
 
 /*
@@ -364,21 +359,32 @@ static int parse_conf(ui_font_config_t *font_config, const char *key,
   vt_font_t font;
   char *font_name;
 
-  if ((font = parse_key(key)) == UNKNOWN_CS) {
+  if ((font = parse_key(key)) == UNKNOWN_CS ||
+      (font_name = alloca(strlen(value) + 1)) == NULL) {
     return 0;
   }
+  strcpy(font_name, value);
 
   /*
-   * XXX Compat with old formats (3.6.3 or before): [entry];[entry];[entry];...
-   *
-   * bl_str_sep() returns NULL only if value == NULL.
+   * XXX
+   * Compat with old formats (3.6.3 or before): [default font name];[font size],[font name];...
    */
-  font_name = bl_str_alloca_dup(value);
-  font_name = bl_str_sep(&font_name, ";");
+#if 1
+  {
+    const char *p = value;
+    while ((p = strchr(p, ';'))) {
+      p++;
+      if (('0' <= *p && *p <= '9') || *p == '\0') {
+        font_name[p - value - 1] = '\0';
 
-  if (parse_value(&font_name, font_name)) {
-    customize_font_name(font_config, font, font_name);
+        break;
+      }
+    }
   }
+#endif
+
+  parse_value(&font_name, font_name);
+  customize_font_name(font_config, font, font_name);
 
   return 1;
 }
@@ -697,6 +703,25 @@ static ui_font_config_t *create_shared_font_config(ui_type_engine_t type_engine,
   return NULL;
 }
 
+static void new_table(ui_font_config_t *font_config) {
+  bl_map_new_with_size(vt_font_t, char *, font_config->font_name_table, bl_map_hash_int,
+                       bl_map_compare_int, 16);
+}
+
+static void destroy_table(ui_font_config_t *font_config) {
+  u_int count;
+  u_int size;
+  BL_PAIR(ui_font_name) * fn_array;
+
+  fn_array = get_font_name_pairs_array(&size, font_config->font_name_table);
+
+  for (count = 0; count < size; count++) {
+    free(fn_array[count]->value);
+  }
+
+  bl_map_destroy(font_config->font_name_table);
+}
+
 /* --- global functions --- */
 
 #if defined(USE_FREETYPE) && defined(USE_FONTCONFIG)
@@ -794,7 +819,7 @@ void ui_release_font_config(ui_font_config_t *font_config) {
     return;
   }
 
-  ui_font_config_delete(font_config);
+  ui_font_config_destroy(font_config);
 
   if (num_configs == 0) {
     free(font_configs);
@@ -809,8 +834,7 @@ ui_font_config_t *ui_font_config_new(ui_type_engine_t type_engine, ui_font_prese
     return NULL;
   }
 
-  bl_map_new_with_size(vt_font_t, char *, font_config->font_name_table, bl_map_hash_int,
-                       bl_map_compare_int, 16);
+  new_table(font_config);
 
   font_config->type_engine = type_engine;
   font_config->font_present = font_present;
@@ -819,19 +843,8 @@ ui_font_config_t *ui_font_config_new(ui_type_engine_t type_engine, ui_font_prese
   return font_config;
 }
 
-void ui_font_config_delete(ui_font_config_t *font_config) {
-  int count;
-  u_int size;
-  BL_PAIR(ui_font_name) * fn_array;
-
-  fn_array = get_font_name_pairs_array(&size, font_config->font_name_table);
-
-  for (count = 0; count < size; count++) {
-    free(fn_array[count]->value);
-  }
-
-  bl_map_delete(font_config->font_name_table);
-
+void ui_font_config_destroy(ui_font_config_t *font_config) {
+  destroy_table(font_config);
   free(font_config);
 }
 
@@ -854,6 +867,7 @@ int ui_customize_font_file(const char *file, /* if null, use "mlterm/font" file.
   ui_font_config_t *targets[6];
   u_int num_targets;
   u_int count;
+  int ret;
 
   if (file == NULL ||
 #if defined(USE_FREETYPE) && defined(USE_FONTCONFIG)
@@ -895,34 +909,39 @@ int ui_customize_font_file(const char *file, /* if null, use "mlterm/font" file.
   }
 #endif
 
-  if (save) {
-    int ret;
+  if (change_custom_cache(file, key, value)) {
+    if (*value == '\0') {
+      /* remove */
+      for (count = 0; count < num_targets; count++) {
+        /*
+         * reset font_name_table
+         *
+         * ~/.mlterm/font: ISO10646_UCS4_1=a
+         * => mlcc font ISO10646_UCS4_1 b
+         * => mlcc font ISO10646_UCS4_1 "" (results in ISO10646_UCS4_1=a)
+         */
+        destroy_table(targets[count]);
+        new_table(targets[count]);
 
-    if (change_custom_cache(file, key, value)) {
+        read_all_conf(targets[count], NULL);
+      }
+
+      return 1;
+    } else {
       ret = 1;
       for (count = 0; count < num_targets; count++) {
         read_all_conf(targets[count], file);
       }
-    } else {
-      ret = 0;
-    }
-
-    if (!save_conf(file, key, value)) {
-      return ret ? -1 : 0;
-    } else {
-      return ret;
     }
   } else {
-    if (!change_custom_cache(file, key, value)) {
-      return 0;
-    }
-
-    for (count = 0; count < num_targets; count++) {
-      read_all_conf(targets[count], file);
-    }
+    ret = 0;
   }
 
-  return 1;
+  if (save && !save_conf(file, key, value)) {
+    return ret ? -1 : 0;
+  } else {
+    return ret;
+  }
 }
 
 char *ui_get_config_font_name(ui_font_config_t *font_config, u_int font_size, vt_font_t font) {
@@ -953,7 +972,7 @@ char *ui_get_config_font_name(ui_font_config_t *font_config, u_int font_size, vt
           ((pair = get_font_name_pair(font_config->font_name_table, FONT_CS(cand_font))) &&
            (font_name = bl_str_replace(pair->value, orig_style[idx], new_style[idx])))) {
 #ifdef DEBUG
-        bl_debug_printf(BL_DEBUG_TAG "Set font %s for %x\n", font_name, font);
+        bl_debug_printf(BL_DEBUG_TAG " Set font %s for %x\n", font_name, font);
 #endif
 
         set_font_name_to_table(font_config->font_name_table, cand_font, font_name);
@@ -1278,7 +1297,7 @@ static void TEST_font_config(void) {
   assert(strcmp("-hoge-bold-i-fuga-", value) == 0);
   free(value);
 
-  ui_font_config_delete(font_config);
+  ui_font_config_destroy(font_config);
 #endif
 }
 
