@@ -5,6 +5,10 @@
 #include <pobl/bl_types.h> /* u_int */
 #include <pobl/bl_str.h>
 
+#ifdef USE_BEOS
+void view_reset_uiwindow(ui_window_t *uiwindow);
+#endif
+
 #ifdef USE_CONSOLE
 static u_int dummy_arg;
 #define SEPARATOR_WIDTH ui_get_opened_displays(&dummy_arg)[0]->display->col_width
@@ -67,7 +71,11 @@ static u_int modify_separator(u_int separator, u_int total, u_int next_min, u_in
   int dont_round_up;
 
   if (total < separator + margin + next_min) {
+#if 0
     separator = total - next_min - margin;
+#else
+    separator = total / 2;
+#endif
     dont_round_up = 1;
   } else {
     dont_round_up = 0;
@@ -391,7 +399,7 @@ static void window_finalized(ui_window_t *win) {
 
   layout = (ui_layout_t *)win;
 
-  ui_layout_delete(layout);
+  ui_layout_destroy(layout);
 }
 
 #ifdef WALL_PICTURE_SIXEL_REPLACES_SYSTEM_PALETTE
@@ -540,25 +548,25 @@ static void set_xdnd_config(ui_window_t *win, char *dev, char *buf, char *value)
 }
 #endif
 
-static void window_deleted_intern(struct terminal *term) {
-  /* dispatch to screen */
-  (*term->screen->window.window_deleted)(&term->screen->window);
-
+static void window_destroyed_intern(struct terminal *term) {
   if (term->next[0]) {
-    window_deleted_intern(term->next[0]);
+    window_destroyed_intern(term->next[0]);
   }
 
   if (term->next[1]) {
-    window_deleted_intern(term->next[1]);
+    window_destroyed_intern(term->next[1]);
   }
+
+  /* dispatch to screen */
+  (*term->screen->window.window_destroyed)(&term->screen->window);
 }
 
-static void window_deleted(ui_window_t *win) {
+static void window_destroyed(ui_window_t *win) {
   ui_layout_t *layout;
 
   layout = (ui_layout_t *)win;
 
-  window_deleted_intern(&layout->term);
+  window_destroyed_intern(&layout->term);
 }
 
 #ifndef USE_QUARTZ
@@ -840,45 +848,76 @@ static u_int total_height(struct terminal *term) {
   return height;
 }
 
-/* XXX */
-static void total_hint_size(struct terminal *term, u_int *width, u_int *height, u_int *width_inc,
-                            u_int *height_inc) {
-  u_int size;
+static void clear_hint_size(struct terminal *term) {
+  int count;
 
-  if (*width_inc < (size = ui_col_width(term->screen))) {
-    *width_inc = size;
+  for (count = 0; count < 2; count++) {
+    if (term->next[count]) {
+      clear_hint_size(term->next[count]);
+    }
   }
 
-  if (*height_inc < (size = ui_line_height(term->screen))) {
-    *height_inc = size;
-  }
+  term->screen->window.sizehint_flag = 0;
+  term->scrollbar.window.sizehint_flag = 0;
+}
 
+static void total_hint_width(struct terminal *term, u_int *width) {
   if (term->sb_mode != SBM_NONE) {
     *width += SEPARATOR_WIDTH;
   }
 
   if (term->next[0]) {
     *width += SEPARATOR_WIDTH;
+    total_hint_width(term->next[0], width);
+  }
 
-    total_hint_size(term->next[0], width, height, width_inc, height_inc);
+  if (term->next[1]) {
+    clear_hint_size(term->next[1]);
+  }
+
+  term->screen->window.sizehint_flag = SIZEHINT_WIDTH;
+  term->scrollbar.window.sizehint_flag = SIZEHINT_WIDTH;
+}
+
+static void total_hint_height(struct terminal *term, u_int *height) {
+  if (term->next[0]) {
+    clear_hint_size(term->next[0]);
   }
 
   if (term->next[1]) {
     *height += SEPARATOR_HEIGHT;
-
-    total_hint_size(term->next[1], width, height, width_inc, height_inc);
+    total_hint_height(term->next[1], height);
   }
+
+  term->screen->window.sizehint_flag = SIZEHINT_HEIGHT;
+  term->scrollbar.window.sizehint_flag = 0;
+}
+
+static void total_hint_size(struct terminal *term, u_int *width, u_int *height) {
+  if (term->sb_mode != SBM_NONE) {
+    *width += SEPARATOR_WIDTH;
+  }
+
+  if (term->next[0]) {
+    *width += SEPARATOR_WIDTH;
+    total_hint_width(term->next[0], width);
+  }
+
+  if (term->next[1]) {
+    *height += SEPARATOR_HEIGHT;
+    total_hint_height(term->next[1], height);
+  }
+
+  term->screen->window.sizehint_flag = SIZEHINT_HEIGHT|SIZEHINT_WIDTH;
+  term->scrollbar.window.sizehint_flag = SIZEHINT_WIDTH;
 }
 
 static void update_normal_hints(ui_layout_t *layout) {
   u_int min_width = 0;
   u_int min_height = 0;
-  u_int width_inc = 0;
-  u_int height_inc = 0;
 
-  total_hint_size(&layout->term, &min_width, &min_height, &width_inc, &height_inc);
-
-  ui_window_set_normal_hints(&layout->window, min_width, min_height, width_inc, height_inc);
+  total_hint_size(&layout->term, &min_width, &min_height);
+  ui_window_set_normal_hints(&layout->window, min_width, min_height, 0, 0);
 }
 
 #ifndef USE_QUARTZ
@@ -974,14 +1013,19 @@ static void change_sb_mode_intern(struct terminal *term, ui_sb_mode_t new_mode,
     goto noresize;
   }
 
-  if ((term->screen->window.x + ACTUAL_WIDTH(&term->screen->window) +
-           (old_mode != SBM_NONE ? SCROLLBAR_WIDTH(term->scrollbar) : 0) <
-       ACTUAL_WIDTH(&layout->window)) ||
-      !ui_window_resize(&layout->window, total_width(&layout->term), total_height(&layout->term),
-                        NOTIFY_TO_MYSELF)) {
-  noresize:
-    reset_layout(&layout->term, 0, 0, layout->window.width, layout->window.height);
+  if (term->screen->window.x + ACTUAL_WIDTH(&term->screen->window) +
+        (old_mode != SBM_NONE ? SCROLLBAR_WIDTH(term->scrollbar) : 0) >=
+      ACTUAL_WIDTH(&layout->window)) {
+    /* This should be called before ui_window_resize(&layout->window) */
+    update_normal_hints(layout);
+    if (ui_window_resize(&layout->window, total_width(&layout->term), total_height(&layout->term),
+                         NOTIFY_TO_MYSELF)) {
+      return;
+    }
   }
+
+noresize:
+  reset_layout(&layout->term, 0, 0, layout->window.width, layout->window.height);
 #endif
 
   update_normal_hints(layout);
@@ -1030,28 +1074,28 @@ static void screen_pointer_motion(ui_window_t *win, XMotionEvent *event) {
   }
 }
 
-static void delete_term(struct terminal *term) {
+static void destroy_term(struct terminal *term) {
   ui_scrollbar_final(&term->scrollbar);
 
   if (term->next[0]) {
-    delete_term(term->next[0]);
+    destroy_term(term->next[0]);
     free(term->next[0]);
   }
 
   if (term->next[1]) {
-    delete_term(term->next[1]);
+    destroy_term(term->next[1]);
     free(term->next[1]);
   }
 }
 
-static void delete_screen_bg_pic(ui_screen_t *screen) {
+static void destroy_screen_bg_pic(ui_screen_t *screen) {
   free(screen->pic_file_path);
   screen->pic_file_path = NULL;
   ui_release_picture(screen->bg_pic);
   screen->bg_pic = NULL;
 }
 
-static void delete_layout_bg_pic(ui_layout_t *layout) {
+static void destroy_layout_bg_pic(ui_layout_t *layout) {
   free(layout->pic_file_path);
   layout->pic_file_path = NULL;
   ui_release_picture(layout->bg_pic);
@@ -1210,7 +1254,7 @@ ui_layout_t *ui_layout_new(ui_screen_t *screen, char *view_name, char *fg_color,
   layout->window.key_pressed = key_pressed;
   layout->window.utf_selection_notified = utf_selection_notified;
   layout->window.xct_selection_notified = xct_selection_notified;
-  layout->window.window_deleted = window_deleted;
+  layout->window.window_destroyed = window_destroyed;
 #ifndef DISABLE_XDND
   layout->window.set_xdnd_config = set_xdnd_config;
 #endif
@@ -1230,12 +1274,12 @@ error:
   return NULL;
 }
 
-void ui_layout_delete(ui_layout_t *layout) {
+void ui_layout_destroy(ui_layout_t *layout) {
   if (layout->bg_pic) {
-    delete_layout_bg_pic(layout);
+    destroy_layout_bg_pic(layout);
   }
 
-  delete_term(&layout->term);
+  destroy_term(&layout->term);
   free(layout);
 }
 
@@ -1352,7 +1396,7 @@ int ui_layout_add_child(ui_layout_t *layout, ui_screen_t *screen, int horizontal
   ui_window_add_child(&layout->window, &screen->window, 0, 0, 0);
 
   if (screen->pic_file_path) {
-    delete_screen_bg_pic(screen);
+    destroy_screen_bg_pic(screen);
   }
 
   ui_window_show(&screen->window, 0);
@@ -1436,18 +1480,31 @@ int ui_layout_remove_child(ui_layout_t *layout, ui_screen_t *screen) {
         term->next[1]->separator_x = term->separator_x;
       }
     }
+
     term->next[idx2]->yfirst = term->yfirst;
 
     next = term->next[idx2];
 
     if (term->next[!idx2]) {
       struct terminal *parent;
+      int empty_idx;
 
+      /* Search from 'next' for a term whose child is NULL. */
       parent = search_parent_term(next, NULL);
       if (parent->next[!idx2] == NULL) {
-        parent->next[!idx2] = term->next[!idx2];
+        empty_idx = !idx2;
       } else {
-        parent->next[idx2] = term->next[!idx2];
+        empty_idx = idx2;
+      }
+
+      parent->next[empty_idx] = term->next[!idx2];
+      if (empty_idx == 0) {
+        parent->separator_x =
+          get_separator_x(parent->screen,
+                          parent->sb_mode == SBM_NONE ? 0 : SCROLLBAR_WIDTH(parent->scrollbar),
+                          100);
+      } else {
+        parent->separator_y = get_separator_y(parent->screen, 100);
       }
     }
 
@@ -1458,6 +1515,9 @@ int ui_layout_remove_child(ui_layout_t *layout, ui_screen_t *screen) {
     term->scrollbar.sb_listener = &term->sb_listener;
 #ifndef USE_QUARTZ
     term->scrollbar.view->win = &term->scrollbar.window;
+#ifdef USE_BEOS
+    view_reset_uiwindow(&term->scrollbar.window);
+#endif
 #endif
 
     term = next;
@@ -1539,7 +1599,7 @@ int ui_layout_remove_child(ui_layout_t *layout, ui_screen_t *screen) {
 
 #ifndef MANAGE_SUB_WINDOWS_BY_MYSELF
   if (ui_screen_attached(screen)) {
-    /* Revert to the original size. */
+    /* Revert the size of vt_term to the original one. */
     ui_window_resize_with_margin(
         &screen->window, ACTUAL_WIDTH(&layout->window) -
                              (term->sb_mode != SBM_NONE ? SCROLLBAR_WIDTH(term->scrollbar) : 0),
