@@ -326,6 +326,8 @@ void ui_prepare_for_main_config(bl_conf_t *conf) {
                   "ignore broadcasted characters [false]");
   bl_conf_add_opt(conf, '\0', "emoji", 0, "emoji_path",
                   "emoji directory or file path [~/.mlterm/emoji]");
+  bl_conf_add_opt(conf, '\0', "lew", 0, "local_echo_wait",
+                  "time (msec) to keep local echo mode [250]");
 #ifdef USE_IM_CURSOR_COLOR
   bl_conf_add_opt(conf, '\0', "imcolor", 0, "im_cursor_color",
                   "cursor color when input method is activated. [false]");
@@ -451,6 +453,7 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
     }
   }
 
+#ifdef USE_XLIB
 #if !defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_CAIRO)
   main_config->type_engine = TYPE_CAIRO;
 #elif !defined(USE_TYPE_XCORE) && defined(USE_TYPE_XFT)
@@ -459,10 +462,11 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
   main_config->type_engine = TYPE_XCORE;
 #endif
 
-#ifdef USE_XLIB
   if ((value = bl_conf_get_value(conf, "type_engine"))) {
     main_config->type_engine = ui_get_type_engine_by_name(value);
   }
+#else
+  main_config->type_engine = TYPE_XCORE;
 #endif
 
 #if !defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XFT) || defined(USE_TYPE_CAIRO) || \
@@ -665,7 +669,7 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
     }
   }
 
-#ifdef __APPLE__
+#if defined(__APPLE__) || defined(USE_BEOS)
   main_config->use_login_shell = 1;
 #else
   main_config->use_login_shell = 0;
@@ -683,7 +687,7 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
 
   main_config->use_mdi = 1;
 
-#ifndef USE_QUARTZ
+#if defined(USE_QUARTZ) || defined(USE_BEOS)
   if ((value = bl_conf_get_value(conf, "use_mdi"))) {
     if (strcmp(value, "false") == 0) {
       main_config->use_mdi = 0;
@@ -691,7 +695,7 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
   }
 #endif
 
-#ifdef USE_CONSOLE
+#if defined(USE_CONSOLE) || defined(COCOA_TOUCH)
   main_config->sb_mode = SBM_NONE;
 #else
   if ((value = bl_conf_get_value(conf, "scrollbar_mode"))) {
@@ -772,8 +776,12 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
     }
   }
 
-  /* "cn" and "ko" ? */
-  if (strcmp(bl_get_lang(), "ja") == 0) {
+  /* "zh" and "ko" ? */
+  if (strcmp(bl_get_lang(), "ja") == 0
+#ifdef USE_WIN32API
+      || strcmp(bl_get_lang(), "Japanese") == 0
+#endif
+     ) {
     main_config->col_size_of_width_a = 2;
   } else {
     main_config->col_size_of_width_a = 1;
@@ -1059,7 +1067,6 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
       main_config->show_dialog = 1;
     }
   }
-
 #endif
 
 #ifdef USE_LIBSSH2
@@ -1200,6 +1207,13 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
     }
   }
 
+  if ((value = bl_conf_get_value(conf, "local_echo_wait"))) {
+    u_int msec;
+    if (bl_str_to_uint(&msec, value)) {
+      vt_set_local_echo_wait(msec);
+    }
+  }
+
   if ((value = bl_conf_get_value(conf, "click_interval"))) {
     int interval;
 
@@ -1351,7 +1365,7 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
 
       if ((main_config->cmd_argv = malloc(sizeof(char *) * (argc + 1) + strlen(value) + 1))) {
         value = strcpy(main_config->cmd_argv + argc + 1, value);
-        _bl_arg_str_to_array(main_config->cmd_argv, &argc, value);
+        bl_arg_str_to_array(main_config->cmd_argv, &argc, value);
         main_config->cmd_path = main_config->cmd_argv[0];
       }
     }
