@@ -13,10 +13,10 @@
 #endif
 
 #include <pobl/bl_debug.h>
-#include <pobl/bl_str.h>  /* bl_str_sep/bl_str_to_int/bl_str_alloca_dup/strdup */
+#include <pobl/bl_str.h>  /* bl_str_sep/bl_str_to_int/strdup */
 #include <pobl/bl_path.h> /* bl_basename */
 #include <pobl/bl_util.h> /* DIGIT_STR_LEN */
-#include <pobl/bl_mem.h>  /* alloca/bl_alloca_garbage_collect/malloc/free */
+#include <pobl/bl_mem.h>  /* alloca/malloc/free */
 #include <pobl/bl_conf.h>
 #include <pobl/bl_conf_io.h>
 #include <pobl/bl_types.h> /* u_int */
@@ -83,7 +83,7 @@ static void config_saved(void) {
   ui_prepare_for_main_config(conf);
   ui_main_config_init(&main_config, conf, 1, argv);
 
-  bl_conf_delete(conf);
+  bl_conf_destroy(conf);
 }
 
 static void font_config_updated(void) {
@@ -206,6 +206,8 @@ static int open_pty_intern(vt_term_t *term, char *cmd_path, char **cmd_argv,
 
 #if defined(USE_WIN32API) || defined(USE_LIBSSH2)
   if (show_dialog || main_config.default_server) {
+    char *default_server_dup;
+    char *uri_dup;
     char *user;
     char *host;
     char *port;
@@ -218,8 +220,9 @@ static int open_pty_intern(vt_term_t *term, char *cmd_path, char **cmd_argv,
 
 #ifdef USE_LIBSSH2
     if (!show_dialog && main_config.default_server &&
+        (default_server_dup = alloca(strlen(main_config.default_server) + 1)) &&
         bl_parse_uri(NULL, &user, &host, &port, NULL, &encoding,
-                     bl_str_alloca_dup(main_config.default_server)) &&
+                     strcpy(default_server_dup, main_config.default_server)) &&
         (session = vt_search_ssh_session(host, port, user))) {
       uri = strdup(main_config.default_server);
       pass = strdup("");
@@ -235,12 +238,15 @@ static int open_pty_intern(vt_term_t *term, char *cmd_path, char **cmd_argv,
       if (vt_get_all_terms(NULL) > 1) {
         return 0;
       }
-    } else {
-      bl_parse_uri(NULL, &user, &host, &port, NULL, &encoding, bl_str_alloca_dup(uri));
+    } else if ((uri_dup = alloca(strlen(uri) + 1))) {
+      bl_parse_uri(NULL, &user, &host, &port, NULL, &encoding, strcpy(uri_dup, uri));
 
 #ifdef USE_LIBSSH2
       vt_pty_ssh_set_use_x11_forwarding(vt_search_ssh_session(host, port, user), x11_fwd);
 #endif
+    } else {
+      /* XXX Not recovering error. */
+      return 0;
     }
 
 #ifdef __DEBUG
@@ -267,15 +273,21 @@ static int open_pty_intern(vt_term_t *term, char *cmd_path, char **cmd_argv,
     }
 
     if (exec_cmd) {
-      int argc;
-      char *tmp;
-
-      tmp = exec_cmd;
-      exec_cmd = bl_str_alloca_dup(exec_cmd);
-      cmd_argv = bl_arg_str_to_array(&argc, exec_cmd);
-      cmd_path = cmd_argv[0];
+      char *tmp = exec_cmd;
 
+      if ((exec_cmd = alloca(strlen(exec_cmd) + 1))) {
+        strcpy(exec_cmd, tmp);
+      }
       free(tmp);
+
+      if ((tmp = bl_argv_alloca(exec_cmd))) {
+        int argc;
+
+        if (bl_arg_str_to_array((char**)tmp, &argc, exec_cmd)) {
+          cmd_argv = (char**)tmp;
+          cmd_path = cmd_argv[0];
+        }
+      }
     }
   }
 #endif
@@ -448,7 +460,7 @@ static void close_screen_win32(ui_screen_t *screen) {
     /*
      * XXX Hack
      * In case SendMessage(WM_CLOSE) causes WM_KILLFOCUS
-     * and operates screen->term which was already deleted.
+     * and operates screen->term which was already destroyed.
      * (see window_unfocused())
      */
     screen->window.window_unfocused = NULL;
@@ -456,8 +468,8 @@ static void close_screen_win32(ui_screen_t *screen) {
 
   SendMessage(ui_get_root_window(&screen->window)->my_window, WM_CLOSE, 0, 0);
 
-  if (is_orphan && screen->window.window_deleted) {
-    (*screen->window.window_deleted)(&screen->window);
+  if (is_orphan && screen->window.window_destroyed) {
+    (*screen->window.window_destroyed)(&screen->window);
   }
 }
 #endif
@@ -468,11 +480,12 @@ static void close_screen_intern(ui_screen_t *screen) {
 
   if (UI_SCREEN_TO_LAYOUT(screen)) {
     ui_layout_remove_child(UI_SCREEN_TO_LAYOUT(screen), screen);
+    /* ui_get_root_window() below doesn't return ui_layout. */
   }
 
   ui_screen_detach(screen);
-  ui_font_manager_delete(screen->font_man);
-  ui_color_manager_delete(screen->color_man);
+  ui_font_manager_destroy(screen->font_man);
+  ui_color_manager_destroy(screen->color_man);
 
   root = ui_get_root_window(&screen->window);
   disp = root->disp;
@@ -616,7 +629,7 @@ static ui_screen_t *open_screen_intern(char *disp_name, vt_term_t *term, ui_layo
   if ((p = realloc(screens, sizeof(ui_screen_t *) * (num_screens + 1))) == NULL) {
     /*
      * XXX
-     * After ui_display_show_root() screen is not deleted correctly by
+     * After ui_display_show_root() screen is not destroyed correctly by
      * 'goto error'(see following error handling in open_pty_intern),
      * but I don't know how to do.
      */
@@ -670,11 +683,11 @@ static ui_screen_t *open_screen_intern(char *disp_name, vt_term_t *term, ui_layo
 
 error:
   if (font_man) {
-    ui_font_manager_delete(font_man);
+    ui_font_manager_destroy(font_man);
   }
 
   if (color_man) {
-    ui_color_manager_delete(color_man);
+    ui_color_manager_destroy(color_man);
   }
 
   if (!root || !ui_display_remove_root(disp, root)) {
@@ -683,11 +696,11 @@ error:
      */
 
     if (screen) {
-      ui_screen_delete(screen);
+      ui_screen_destroy(screen);
     }
 
     if (layout) {
-      ui_layout_delete(layout);
+      ui_layout_destroy(layout);
     }
   }
 
@@ -731,7 +744,6 @@ static void __exit(void *p, int status) {
 
   bl_msg_printf("reporting unfreed memories --->\n");
 
-  bl_alloca_garbage_collect();
   bl_mem_free_all();
 
   bl_dl_close_all();
@@ -752,6 +764,7 @@ static void open_pty(void *p, ui_screen_t *screen, char *dev) {
 #if defined(USE_WIN32API) || defined(USE_LIBSSH2)
     char *default_server;
     char *new_cmd_line;
+    char *new_cmd_line_dup;
     char *cmd_path;
     char **cmd_argv;
 #endif
@@ -773,14 +786,19 @@ static void open_pty(void *p, ui_screen_t *screen, char *dev) {
     }
 
     if ((new_cmd_line = vt_term_get_cmd_line(screen->term)) &&
-        (new_cmd_line = bl_str_alloca_dup(new_cmd_line))) {
+        (new_cmd_line_dup = alloca(strlen(new_cmd_line) + 1))) {
       int argc;
 
       cmd_path = main_config.cmd_path;
       cmd_argv = main_config.cmd_argv;
 
-      main_config.cmd_argv = bl_arg_str_to_array(&argc, new_cmd_line);
-      main_config.cmd_path = main_config.cmd_argv[0];
+      if ((main_config.cmd_argv = bl_argv_alloca(new_cmd_line)) &&
+          bl_arg_str_to_array(main_config.cmd_argv, &argc,
+                              strcpy(new_cmd_line_dup, new_cmd_line))) {
+        main_config.cmd_path = main_config.cmd_argv[0];
+      } else {
+        main_config.cmd_argv = cmd_argv;
+      }
     }
 #endif
 
@@ -863,7 +881,7 @@ static void close_pty(void *p, ui_screen_t *screen, char *dev) {
   bl_trigger_sig_child(vt_term_get_child_pid(term));
 }
 
-static void pty_closed(void *p, ui_screen_t *screen /* screen->term was already deleted. */
+static void pty_closed(void *p, ui_screen_t *screen /* screen->term was already destroyed. */
                        ) {
   int count;
 
@@ -876,6 +894,24 @@ static void pty_closed(void *p, ui_screen_t *screen /* screen->term was already
       vt_term_t *term;
 
       if ((term = vt_get_detached_term(NULL)) == NULL) {
+#ifdef COCOA_TOUCH
+        if (vt_get_all_terms(NULL) == 0 && (term = create_term_intern())) {
+          if (open_pty_intern(term, main_config.cmd_path, main_config.cmd_argv, &screen->window,
+#ifdef USE_LIBSSH2
+                               main_config.show_dialog
+#else
+                               0
+#endif
+                               )) {
+            ui_screen_attach(screen, term);
+
+            return;
+          } else {
+            vt_destroy_term(term);
+          }
+        }
+#endif
+
 #ifdef __DEBUG
         bl_debug_printf(" no detached term. closing screen.\n");
 #endif
@@ -905,6 +941,7 @@ static void open_cloned_screen(ui_screen_t *cur_screen, ui_layout_t *layout, int
 #if defined(USE_WIN32API) || defined(USE_LIBSSH2)
   char *default_server;
   char *new_cmd_line;
+  char *new_cmd_line_dup;
   char *cmd_path;
   char **cmd_argv;
 #endif
@@ -918,14 +955,19 @@ static void open_cloned_screen(ui_screen_t *cur_screen, ui_layout_t *layout, int
   }
 
   if ((new_cmd_line = vt_term_get_cmd_line(cur_screen->term)) &&
-      (new_cmd_line = bl_str_alloca_dup(new_cmd_line))) {
+      (new_cmd_line_dup = alloca(strlen(new_cmd_line) + 1))) {
     int argc;
 
     cmd_path = main_config.cmd_path;
     cmd_argv = main_config.cmd_argv;
 
-    main_config.cmd_argv = bl_arg_str_to_array(&argc, new_cmd_line);
-    main_config.cmd_path = main_config.cmd_argv[0];
+    if ((main_config.cmd_argv = bl_argv_alloca(new_cmd_line)) &&
+        bl_arg_str_to_array(main_config.cmd_argv, &argc,
+                            strcpy(new_cmd_line_dup, new_cmd_line))) {
+      main_config.cmd_path = main_config.cmd_argv[0];
+    } else {
+      main_config.cmd_argv = cmd_argv;
+    }
   }
 #endif
 
@@ -994,8 +1036,18 @@ static int close_screen(void *p, ui_screen_t *screen, /* Screen which triggers t
     bl_debug_printf(BL_DEBUG_TAG " screen %p is registered to be closed.\n", screen);
 #endif
 
+#ifdef USE_BEOS
+    /*
+     * Don't use ui_close_dead_screens().
+     * It doesn't clear dead_mask because BWindow::Quit() called from
+     * close_screen_intern() exists the current thread.
+     */
+    screens[count] = screens[--num_screens];
+    close_screen_intern(screen);
+#else
     idx = count / MSU; /* count / 8 */
     dead_mask[idx] |= (1 << (count - MSU * idx));
+#endif
 
     break;
   }
@@ -1033,7 +1085,9 @@ static int mlclient(void *self, ui_screen_t *screen, char *args,
   char **argv;
   int argc;
 
-  argv = bl_arg_str_to_array(&argc, args);
+  if (!(argv = bl_argv_alloca(args)) || !bl_arg_str_to_array(argv, &argc, args)) {
+    return 0;
+  }
 
 #ifdef __DEBUG
   {
@@ -1104,22 +1158,25 @@ static int mlclient(void *self, ui_screen_t *screen, char *args,
     bl_conf_add_opt(conf, '\0', "vsep", 0, "vsep", "");
 
     if (!bl_conf_parse_args(conf, &argc, &argv, 1)) {
-      bl_conf_delete(conf);
+      bl_conf_destroy(conf);
 
       return 0;
     }
 
     if (screen && UI_SCREEN_TO_LAYOUT(screen)) {
-      if ((sep = bl_conf_get_value(conf, "hsep"))) {
+      char *p;
+
+      if ((p = bl_conf_get_value(conf, "hsep"))) {
         horizontal = 1;
-      }
-      else if ((sep = bl_conf_get_value(conf, "vsep"))) {
+      } else if ((p = bl_conf_get_value(conf, "vsep"))) {
         horizontal = 0;
-      }
-      else {
+      } else {
+        sep = NULL;
         goto end_check_sep;
       }
-      sep = bl_str_alloca_dup(sep);
+      if ((sep = alloca(strlen(p) + 1))) {
+        strcpy(sep, p);
+      }
     } else {
       sep = NULL;
     }
@@ -1129,7 +1186,7 @@ static int mlclient(void *self, ui_screen_t *screen, char *args,
 
     ui_main_config_init(&main_config, conf, argc, argv);
 
-    bl_conf_delete(conf);
+    bl_conf_destroy(conf);
 
     if (screen) {
       if (sep) {
