@@ -58,6 +58,8 @@ static int parse(int *protoid, /* If seq doesn't have proto, -1 is set. */
   if (proto) {
     if (strcmp(proto, "ssh") == 0) {
       *protoid = IDD_SSH;
+    } else if (strcmp(proto, "mosh") == 0) {
+      *protoid = IDD_MOSH;
     } else if (strcmp(proto, "telnet") == 0) {
       *protoid = IDD_TELNET;
     } else if (strcmp(proto, "rlogin") == 0) {
@@ -164,6 +166,7 @@ LRESULT CALLBACK dialog_proc(HWND dlgwin, UINT msg, WPARAM wparam, LPARAM lparam
 #endif
 
       if (default_server) {
+        char *default_server_dup;
         char *user;
         int proto;
         char *server;
@@ -177,7 +180,9 @@ LRESULT CALLBACK dialog_proc(HWND dlgwin, UINT msg, WPARAM wparam, LPARAM lparam
         }
 #endif
 
-        if (parse(&proto, &user, &server, &port, &encoding, bl_str_alloca_dup(default_server))) {
+        if ((default_server_dup = alloca(strlen(default_server) + 1)) &&
+            parse(&proto, &user, &server, &port, &encoding,
+                  strcpy(default_server_dup, default_server))) {
           SetWindowText(GetDlgItem(dlgwin, IDD_SERVER), server);
           item = IDD_USER;
 
@@ -190,7 +195,11 @@ LRESULT CALLBACK dialog_proc(HWND dlgwin, UINT msg, WPARAM wparam, LPARAM lparam
             item = IDD_PASS;
           }
 
-#ifndef USE_LIBSSH2
+#ifdef USE_LIBSSH2
+          if (proto == IDD_SSH || proto == IDD_MOSH) {
+            selected_proto = proto;
+          }
+#else
           if (proto != -1) {
             selected_proto = proto;
           }
@@ -207,10 +216,8 @@ LRESULT CALLBACK dialog_proc(HWND dlgwin, UINT msg, WPARAM wparam, LPARAM lparam
 #ifdef USE_LIBSSH2
       EnableWindow(GetDlgItem(dlgwin, IDD_TELNET), FALSE);
       EnableWindow(GetDlgItem(dlgwin, IDD_RLOGIN), FALSE);
-      CheckRadioButton(dlgwin, IDD_SSH, IDD_RLOGIN, IDD_SSH);
-#else
-      CheckRadioButton(dlgwin, IDD_SSH, IDD_RLOGIN, selected_proto);
 #endif
+      CheckRadioButton(dlgwin, IDD_SSH, IDD_RLOGIN, selected_proto);
 
 #ifdef USE_LIBSSH2
       if (use_x11_forwarding) {
@@ -258,6 +265,7 @@ LRESULT CALLBACK dialog_proc(HWND dlgwin, UINT msg, WPARAM wparam, LPARAM lparam
       break;
 
     case IDD_SSH:
+    case IDD_MOSH:
     case IDD_TELNET:
     case IDD_RLOGIN:
       selected_proto = LOWORD(wparam);
@@ -338,6 +346,14 @@ static char *exec_servman(char *server, DWORD server_len) {
    * use the wShowWindow flags.
    */
 
+  /*
+   * Win32 environmental variables are cleared in a cygwin program.
+   * servman.exe (which works without cygwin APIs) needs HOMEPATH environmental variable.
+   */
+#ifndef USE_WIN32API
+  SetEnvironmentVariable("HOMEPATH", getenv("HOMEPATH"));
+#endif
+
   if (!CreateProcess("servman.exe", "servman.exe", NULL, NULL, TRUE, 0, NULL, NULL, &si, &pi)) {
 #ifdef DEBUG
     bl_warn_printf(BL_DEBUG_TAG " CreateProcess() failed.\n");
@@ -439,6 +455,8 @@ int ui_connect_dialog(char **uri,      /* Should be free'ed by those who call th
     goto end;
   } else if (selected_proto == IDD_SSH) {
     proto = "ssh://";
+  } else if (selected_proto == IDD_MOSH) {
+    proto = "mosh://";
   } else if (selected_proto == IDD_TELNET) {
     proto = "telnet://";
   } else if (selected_proto == IDD_RLOGIN) {
