@@ -354,15 +354,16 @@ static int is_in_the_same_window_family(ui_window_t *win, Window window) {
 }
 #endif
 
+/* RIGHT_MARGIN is not added. (See WM_SIZE) */
 static u_int total_min_width(ui_window_t *win) {
-  int count;
+  u_int count;
   u_int min_width;
 
   min_width = win->min_width + win->hmargin * 2;
 
   for (count = 0; count < win->num_children; count++) {
-    if (win->children[count]->is_mapped) {
-      /* XXX */
+    if (win->children[count]->is_mapped &&
+        (win->children[count]->sizehint_flag & SIZEHINT_WIDTH)) {
       min_width += total_min_width(win->children[count]);
     }
   }
@@ -370,15 +371,16 @@ static u_int total_min_width(ui_window_t *win) {
   return min_width;
 }
 
+/* BOTTOM_MARGIN is not added. (See WM_SIZE) */
 static u_int total_min_height(ui_window_t *win) {
-  int count;
+  u_int count;
   u_int min_height;
 
   min_height = win->min_height + win->vmargin * 2;
 
   for (count = 0; count < win->num_children; count++) {
-    if (win->children[count]->is_mapped) {
-      /* XXX */
+    if (win->children[count]->is_mapped &&
+        (win->children[count]->sizehint_flag & SIZEHINT_HEIGHT)) {
       min_height += total_min_height(win->children[count]);
     }
   }
@@ -387,13 +389,14 @@ static u_int total_min_height(ui_window_t *win) {
 }
 
 static u_int total_width_inc(ui_window_t *win) {
-  int count;
+  u_int count;
   u_int width_inc;
 
   width_inc = win->width_inc;
 
   for (count = 0; count < win->num_children; count++) {
-    if (win->children[count]->is_mapped) {
+    if (win->children[count]->is_mapped &&
+        (win->children[count]->sizehint_flag & SIZEHINT_WIDTH)) {
       u_int sub_inc;
 
       /*
@@ -410,13 +413,14 @@ static u_int total_width_inc(ui_window_t *win) {
 }
 
 static u_int total_height_inc(ui_window_t *win) {
-  int count;
+  u_int count;
   u_int height_inc;
 
   height_inc = win->height_inc;
 
   for (count = 0; count < win->num_children; count++) {
-    if (win->children[count]->is_mapped) {
+    if (win->children[count]->is_mapped &&
+        (win->children[count]->sizehint_flag & SIZEHINT_HEIGHT)) {
       u_int sub_inc;
 
       /*
@@ -797,6 +801,7 @@ int ui_window_init(ui_window_t *win, u_int width, u_int height, u_int min_width,
   win->min_height = min_height;
   win->width_inc = width_inc;
   win->height_inc = height_inc;
+  win->sizehint_flag = SIZEHINT_WIDTH|SIZEHINT_HEIGHT;
   win->hmargin = hmargin;
   win->vmargin = vmargin;
 
@@ -832,7 +837,7 @@ void ui_window_final(ui_window_t *win) {
 
   /*
    * DestroyWindow() is not called here because DestroyWindow internally sends
-   * WM_DESTROY message which causes window_deleted event again.
+   * WM_DESTROY message which causes window_destroyed event again.
    * If you want to close window, call SendMessage( WM_CLOSE ) instead of
    * ui_window_final().
    */
@@ -840,7 +845,7 @@ void ui_window_final(ui_window_t *win) {
   ui_xic_deactivate(win);
 
 #if 0
-  (*m_cp_parser.delete)(&m_cp_parser);
+  (*m_cp_parser.destroy)(&m_cp_parser);
 #endif
 
   if (win->window_finalized) {
@@ -1498,8 +1503,8 @@ int ui_window_receive_event(ui_window_t *win, XEvent *event) {
 
   switch (event->msg) {
     case WM_DESTROY:
-      if (win->window_deleted) {
-        (*win->window_deleted)(win);
+      if (win->window_destroyed) {
+        (*win->window_destroyed)(win);
       }
 
       return 1;
@@ -2104,10 +2109,9 @@ int ui_window_receive_event(ui_window_t *win, XEvent *event) {
         min_width = total_min_width(win);
         min_height = total_min_height(win);
 
-        if (width < min_width + win->hmargin * 2 || height < min_height + win->vmargin * 2) {
-          ui_window_resize(win, BL_MAX(min_width + win->hmargin * 2, width) - win->hmargin * 2,
-                           BL_MAX(min_height + win->vmargin * 2, height) - win->vmargin * 2,
-                           NOTIFY_TO_MYSELF);
+        if (width < min_width || height < min_height) {
+          ui_window_resize(win, BL_MAX(min_width, width) - win->hmargin * 2,
+                           BL_MAX(min_height, height) - win->vmargin * 2, NOTIFY_TO_MYSELF);
         } else if (width != ACTUAL_WIDTH(win) || height != ACTUAL_HEIGHT(win)) {
           u_int width_surplus;
           u_int height_surplus;
@@ -2116,7 +2120,6 @@ int ui_window_receive_event(ui_window_t *win, XEvent *event) {
             width_surplus = height_surplus = 0;
           } else {
             width_surplus = (width - min_width - win->hmargin * 2) % total_width_inc(win);
-
             height_surplus = (height - min_height - win->vmargin * 2) % total_height_inc(win);
           }
 
@@ -2596,7 +2599,7 @@ void ui_set_window_name(ui_window_t *win, u_char *name) {
       len = (*utf_conv->convert)(utf_conv, buf, len * 4, parser);
     }
 
-    (*parser->delete)(parser);
+    (*parser->destroy)(parser);
 
     buf[len] = '\0';
     buf[len + 1] = '\0';
