@@ -3,7 +3,6 @@
 #include "../ui_xic.h"
 
 #include <pobl/bl_debug.h>
-#include <pobl/bl_str.h>    /* bl_str_alloca_dup */
 #include <pobl/bl_mem.h>    /* malloc */
 #include <pobl/bl_locale.h> /* bl_get_locale */
 
@@ -81,7 +80,7 @@ int ui_xic_deactivate(ui_window_t *win) {
   }
 
   ImmReleaseContext(win->my_window, win->xic->ic);
-  (*win->xic->parser->delete)(win->xic->parser);
+  (*win->xic->parser->destroy)(win->xic->parser);
   free(win->xic);
   win->xic = NULL;
 
