@@ -13,6 +13,7 @@
 #include <pobl/bl_dialog.h>
 #include <pobl/bl_net.h>     /* getaddrinfo/socket/connect/sockaddr_un */
 #include <pobl/bl_conf_io.h> /* bl_get_user_rc_path */
+#include <pobl/bl_util.h> /* bl_get_user_name */
 
 #if !defined(USE_WIN32API) && defined(HAVE_PTHREAD)
 #include <pthread.h>
@@ -25,6 +26,10 @@
 #include <windows.h> /* GetModuleHandle */
 #endif
 
+#ifdef __HAIKU__
+#include <sys/select.h> /* fd_set */
+#endif
+
 #ifndef USE_WIN32API
 #define closesocket(sock) close(sock)
 #endif
@@ -217,7 +222,7 @@ static void kbd_callback(const char *name, int name_len, const char *instruction
 }
 
 #ifdef BL_DEBUG
-#define strdup(str) bl_str_dup(str, __FILE__, __LINE__, __FUNCTION__)
+#define strdup(str) __bl_str_dup(str, __FILE__, __LINE__, __FUNCTION__)
 #endif
 
 #ifdef OPEN_PTY_ASYNC
@@ -264,7 +269,12 @@ static void set_use_multi_thread(int use) {
 #ifdef USE_WIN32API
         openssl_locks[count] = CreateMutex(NULL, FALSE, NULL);
 #else
-        openssl_locks[count] = PTHREAD_MUTEX_INITIALIZER;
+        /*
+         * "openssl_locks[count] = PTHREAD_MUTEX_INITIALIZER" fails in some environments
+         * such as HaikuOS(x86, gcc-2.95.3)
+         */
+        pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
+        openssl_locks[count] = mutex;
 #endif
       }
 
@@ -520,7 +530,19 @@ static ssh_session_t *ssh_connect(const char *host, const char *port, const char
   }
 
   if (!(userauthlist = libssh2_userauth_list(session->obj, user, strlen(user)))) {
-    goto error4;
+#ifdef DEBUG
+    bl_debug_printf(BL_DEBUG_TAG " No authentication method.\n");
+#endif
+
+    if (libssh2_session_last_errno(session->obj) == 0) {
+#ifdef DEBUG
+    bl_debug_printf(BL_DEBUG_TAG " Skip authentication.\n");
+#endif
+
+      goto auth_end;
+    } else {
+      goto error4;
+    }
   }
 
 #ifdef __DEBUG
@@ -664,6 +686,7 @@ static ssh_session_t *ssh_connect(const char *host, const char *port, const char
 #endif
   }
 
+auth_end:
   {
     void *p;
 
@@ -713,7 +736,6 @@ static int ssh_disconnect(ssh_session_t *session) {
   u_int count;
 
   if (session->num_ptys > 0) {
-    /* In case this function is called from vt_pty_new. */
     libssh2_session_set_blocking(session->obj, 0);
 
     return 0;
@@ -873,8 +895,8 @@ static ssize_t read_pty(vt_pty_t *pty, u_char *buf, size_t len) {
   SetEvent(rd_ev);
 #endif
 
-  if (ret == LIBSSH2_ERROR_SOCKET_SEND || ret == LIBSSH2_ERROR_SOCKET_RECV ||
-      libssh2_channel_eof(((vt_pty_ssh_t *)pty)->channel)) {
+  if (ret <= 0 && (ret == LIBSSH2_ERROR_SOCKET_SEND || ret == LIBSSH2_ERROR_SOCKET_RECV ||
+                   libssh2_channel_eof(((vt_pty_ssh_t *)pty)->channel))) {
     if ((ret < 0 && reconnect((vt_pty_ssh_t*)pty)) || zombie((vt_pty_ssh_t *)pty)) {
       return 0;
     }
@@ -894,10 +916,12 @@ static int scp_stop(vt_pty_ssh_t *pty_ssh) {
 }
 
 #ifdef USE_WIN32API
+/* Same as vt_pty_mosh.cpp */
 static ssize_t lo_recv_pty(vt_pty_t *pty, u_char *buf, size_t len) {
   return recv(pty->master, buf, len, 0);
 }
 
+/* Same as vt_pty_mosh.cpp */
 static ssize_t lo_send_to_pty(vt_pty_t *pty, u_char *buf, size_t len) {
   if (len == 1 && buf[0] == '\x03') {
     /* ^C */
@@ -912,12 +936,13 @@ static ssize_t lo_send_to_pty(vt_pty_t *pty, u_char *buf, size_t len) {
   return send(pty->slave, buf, len, 0);
 }
 
+/* Same as vt_pty_mosh.cpp */
 static int _socketpair(int af, int type, int proto, SOCKET sock[2]) {
   SOCKET listen_sock;
   SOCKADDR_IN addr;
   int addr_len;
 
-  if ((listen_sock = WSASocket(af, type, proto, NULL, 0, 0)) == -1) {
+  if ((listen_sock = WSASocket(af, type, proto, NULL, 0, 0)) == INVALID_SOCKET) {
     return -1;
   }
 
@@ -941,7 +966,7 @@ static int _socketpair(int af, int type, int proto, SOCKET sock[2]) {
   }
 
   /* select() and receive() can call simultaneously on java. */
-  if ((sock[0] = WSASocket(af, type, proto, NULL, 0, WSA_FLAG_OVERLAPPED)) == -1) {
+  if ((sock[0] = WSASocket(af, type, proto, NULL, 0, WSA_FLAG_OVERLAPPED)) == INVALID_SOCKET) {
     goto error1;
   }
 
@@ -985,7 +1010,7 @@ static int check_sig_child(pid_t pid) {
 static ssize_t lo_read_pty(vt_pty_t *pty, u_char *buf, size_t len) {
 #ifdef __CYGWIN__
   if (check_sig_child(pty->config_menu.pid)) {
-    /* vt_pty_set_use_loopback(0) is called from sig_child() in vt_config_menu.c. */
+    /* vt_pty_ssh_set_use_loopback(0) is called from sig_child() in vt_config_menu.c. */
     return 0;
   }
 #endif
@@ -997,7 +1022,7 @@ static ssize_t lo_write_to_pty(vt_pty_t *pty, u_char *buf, size_t len) {
 #ifdef __CYGWIN__
   if (check_sig_child(pty->config_menu.pid)) {
     /*
-     * vt_pty_set_use_loopback(0) is called from sig_child()
+     * vt_pty_ssh_set_use_loopback(0) is called from sig_child()
      * in vt_config_menu.c is called
      */
     return 0;
@@ -1081,7 +1106,7 @@ static int unuse_loopback(vt_pty_t *pty) {
   ssize_t len;
 
   if (!pty->stored || --(pty->stored->ref_count) > 0) {
-    return 1;
+    return 0;
   }
 
   while ((len = (*pty->read)(pty, buf, sizeof(buf))) > 0) {
@@ -1249,16 +1274,17 @@ static int setup_x11(LIBSSH2_CHANNEL *channel) {
   }
 
   if (strncmp(display, "unix:", 5) == 0) {
-    display_port_str = display + 5;
+    p = display + 5;
   } else if (display[0] == ':') {
-    display_port_str = display + 1;
+    p = display + 1;
   } else {
     return 0;
   }
 
-  if (!(display_port_str = bl_str_alloca_dup(display_port_str))) {
+  if (!(display_port_str = alloca(strlen(p) + 1))) {
     return 0;
   }
+  strcpy(display_port_str, p);
 
   if ((p = strrchr(display_port_str, '.'))) {
     *p = '\0';
@@ -1492,6 +1518,7 @@ static int open_channel(vt_pty_ssh_t *pty,    /* pty->session is non-blocking */
   pty->pty.set_winsize = set_winsize;
   pty->pty.write = write_to_pty;
   pty->pty.read = read_pty;
+  pty->pty.mode = PTY_SSH;
 
   if (set_winsize(&pty->pty, cols, rows, width_pix, height_pix) == 0) {
 #ifdef DEBUG
@@ -1863,21 +1890,10 @@ static void save_data_for_reconnect(ssh_session_t *session, const char *cmd_path
   }
 }
 
-static char *get_user_name(void) {
-  char *user;
-
-  /* USER/LOGNAME: unix , USERNAME: win32 */
-  if (!(user = getenv("USER")) && !(user = getenv("USERNAME"))) {
-    user = getenv("LOGNAME");
-  }
-
-  return user;
-}
-
 /* --- global functions --- */
 
 #ifdef USE_WIN32API
-void vt_pty_set_pty_read_trigger(void (*func)(void)) {
+void vt_pty_ssh_set_pty_read_trigger(void (*func)(void)) {
   trigger_pty_read = func;
 }
 #endif
@@ -1892,16 +1908,18 @@ vt_pty_t *vt_pty_ssh_new(const char *cmd_path, /* can be NULL */
                          const char *privkey,                                   /* can be NULL */
                          u_int cols, u_int rows, u_int width_pix, u_int height_pix) {
   vt_pty_ssh_t *pty;
+  char *uri_dup;
   char *user;
   char *proto;
   char *host;
   char *port;
 
-  if (!bl_parse_uri(&proto, &user, &host, &port, NULL, NULL, bl_str_alloca_dup(uri))) {
+  if ((uri_dup = alloca(strlen(uri) + 1)) == NULL ||
+      !bl_parse_uri(&proto, &user, &host, &port, NULL, NULL, strcpy(uri_dup, uri))) {
     return NULL;
   }
 
-  if (!user && !(user = get_user_name())) {
+  if (!user && !(user = bl_get_user_name())) {
     return NULL;
   }
 
@@ -1934,7 +1952,7 @@ void *vt_search_ssh_session(const char *host, const char *port, /* can be NULL *
                             ) {
   int count;
 
-  if (!user && !(user = get_user_name())) {
+  if (!user && !(user = bl_get_user_name())) {
     return NULL;
   }
 
@@ -1954,7 +1972,7 @@ void *vt_search_ssh_session(const char *host, const char *port, /* can be NULL *
   return NULL;
 }
 
-int vt_pty_set_use_loopback(vt_pty_t *pty, int use) {
+int vt_pty_ssh_set_use_loopback(vt_pty_t *pty, int use) {
   if (use) {
     if (((vt_pty_ssh_t *)pty)->session->suspended) {
       return 0;
