@@ -12,7 +12,7 @@
 #include <pobl/bl_debug.h>
 #include <pobl/bl_sig_child.h>
 #include <pobl/bl_mem.h> /* alloca */
-#include <pobl/bl_str.h> /* bl_str_alloca_dup */
+#include <pobl/bl_str.h> /* bl_str_to_uint */
 #include <pobl/bl_conf.h>
 #include <pobl/bl_conf_io.h>
 #include <pobl/bl_path.h>   /* bl_basename */
@@ -91,10 +91,10 @@ vt_term_t *mlterm_open(char *host, char *pass, int cols, int rows, u_int log_siz
     CreateEvent(NULL, FALSE, FALSE, "PTY_READ_READY");
 #endif
 
-    bl_init_prog("mlterm", "3.8.6");
+    bl_init_prog("mlterm", "3.8.8");
     bl_set_sys_conf_dir(CONFIG_PATH);
     bl_locale_init("");
-    bl_sig_child_init();
+    bl_sig_child_start();
     vt_term_manager_init(1);
     vt_color_config_init();
 
@@ -286,11 +286,11 @@ vt_term_t *mlterm_open(char *host, char *pass, int cols, int rows, u_int log_siz
           (default_argv = malloc(sizeof(char *) * bl_count_char_in_str(value, ' ') + 2))) {
         int argc;
 
-        _bl_arg_str_to_array(default_argv, &argc, strdup(value));
+        bl_arg_str_to_array(default_argv, &argc, strdup(value));
         default_cmd_path = default_argv[0];
       }
 
-      bl_conf_delete(conf);
+      bl_conf_destroy(conf);
     }
 
     unicode_policy |= ONLY_USE_UNICODE_BOXDRAW_FONT;
