@@ -84,12 +84,9 @@ static int insert_chars(vt_edit_t *edit, vt_char_t *ins_chars, u_int num_ins_cha
   }
 
   if ((buffer = vt_str_alloca(buf_len)) == NULL) {
-#ifdef DEBUG
-    bl_warn_printf(BL_DEBUG_TAG " alloca() failed.\n");
-#endif
-
     return 0;
   }
+  vt_str_init(buffer, buf_len);
 
   filled_len = 0;
   filled_cols = 0;
@@ -582,7 +579,6 @@ int vt_edit_clone(vt_edit_t *dst_edit, vt_edit_t *src_edit) {
 }
 
 int vt_edit_resize(vt_edit_t *edit, u_int num_cols, u_int num_rows) {
-  u_int old_filled_rows;
   u_int old_cols;
   u_int slide;
 
@@ -590,10 +586,27 @@ int vt_edit_resize(vt_edit_t *edit, u_int num_cols, u_int num_rows) {
   vt_cursor_dump(&edit->cursor);
 #endif
 
-  if ((old_filled_rows = vt_model_get_num_filled_rows(&edit->model)) > num_rows) {
-    if (edit->is_logging && edit->scroll_listener->receive_scrolled_out_line) {
-      int count;
+  if (edit->is_logging && edit->scroll_listener->receive_scrolled_out_line) {
+    u_int count;
+    u_int old_filled_rows = vt_model_get_num_filled_rows(&edit->model);
+    int scroll_all = 0;
 
+    for (count = 0; count < old_filled_rows; count++) {
+      vt_line_t *line = vt_model_get_line(&edit->model, count);
+      if (vt_str_cols(line->chars, vt_line_get_num_filled_chars_except_sp(line)) > num_cols) {
+        scroll_all = 1;
+        break;
+      }
+    }
+
+    if (scroll_all) {
+      for (count = 0; count < old_filled_rows; count++) {
+        (*edit->scroll_listener->receive_scrolled_out_line)(edit->scroll_listener->self,
+                                                            vt_model_get_line(&edit->model, count));
+      }
+      vt_edit_goto_home(edit);
+      vt_edit_clear_below(edit);
+    } else if (old_filled_rows > num_rows) {
       for (count = 0; count < old_filled_rows - num_rows; count++) {
         (*edit->scroll_listener->receive_scrolled_out_line)(edit->scroll_listener->self,
                                                             vt_model_get_line(&edit->model, count));
@@ -689,12 +702,9 @@ int vt_edit_insert_blank_chars(vt_edit_t *edit, u_int num_blank_chars) {
   reset_wraparound_checker(edit);
 
   if ((blank_chars = vt_str_alloca(num_blank_chars)) == NULL) {
-#ifdef DEBUG
-    bl_warn_printf(BL_DEBUG_TAG " alloca() failed.\n");
-#endif
-
     return 0;
   }
+  vt_str_init(blank_chars, num_blank_chars);
 
   if (edit->use_bce) {
     /*
@@ -740,12 +750,9 @@ int vt_edit_overwrite_chars(vt_edit_t *edit, vt_char_t *ow_chars, u_int num_ow_c
   }
 
   if ((buffer = vt_str_alloca(buf_len)) == NULL) {
-#ifdef DEBUG
-    bl_warn_printf(BL_DEBUG_TAG " alloca() failed.\n");
-#endif
-
     return 0;
   }
+  vt_str_init(buffer, buf_len);
 
   line = CURSOR_LINE(edit);
   filled_len = 0;
@@ -901,12 +908,9 @@ int vt_edit_delete_cols(vt_edit_t *edit, u_int del_cols) {
   buf_len = cursor_line->num_chars - edit->cursor.col;
 
   if ((buffer = vt_str_alloca(buf_len)) == NULL) {
-#ifdef DEBUG
-    bl_warn_printf(BL_DEBUG_TAG " alloca() failed.\n");
-#endif
-
     return 0;
   }
+  vt_str_init(buffer, buf_len);
 
   filled_len = 0;
 
