@@ -256,7 +256,7 @@ static int vt_line_set_use_ot_layout(vt_line_t *line, int flag) {
     line->ctl_info_type = VINFO_OT_LAYOUT;
   } else {
     if (vt_line_is_using_ot_layout(line)) {
-      vt_ot_layout_delete(line->ctl_info.ot_layout);
+      vt_ot_layout_destroy(line->ctl_info.ot_layout);
       line->ctl_info_type = 0;
     }
   }
@@ -285,6 +285,7 @@ static int vt_line_ot_layout_visual(vt_line_t *line) {
   if ((src = vt_str_alloca(src_len)) == NULL) {
     return 0;
   }
+  vt_str_init(src, src_len);
 
   vt_str_copy(src, line->chars, src_len);
 
@@ -294,7 +295,7 @@ static int vt_line_ot_layout_visual(vt_line_t *line) {
 
     if ((chars = vt_str_new(dst_len))) {
       /* XXX => shrunk at vt_screen.c and vt_logical_visual_ctl.c */
-      vt_str_delete(line->chars, line->num_chars);
+      vt_str_destroy(line->chars, line->num_chars);
       line->chars = chars;
       line->num_chars = dst_len;
     } else {
@@ -376,6 +377,7 @@ static int vt_line_ot_layout_logical(vt_line_t *line) {
   if ((src = vt_str_alloca(src_len)) == NULL) {
     return 0;
   }
+  vt_str_init(src, src_len);
 
   vt_str_copy(src, line->chars, src_len);
   dst = line->chars;
@@ -663,7 +665,7 @@ void vt_line_final(vt_line_t *line) {
 #endif
 
   if (line->chars) {
-    vt_str_delete(line->chars, line->num_chars);
+    vt_str_destroy(line->chars, line->num_chars);
   }
 }
 
@@ -1234,14 +1236,6 @@ int vt_convert_char_index_to_col(vt_line_t *line, int char_index, int flag /* BR
 
   if ((flag & BREAK_BOUNDARY) && line->num_filled_chars <= char_index) {
     for (count = 0; count < line->num_filled_chars; count++) {
-#ifdef DEBUG
-      if (vt_char_cols(line->chars + count) == 0) {
-        bl_warn_printf(BL_DEBUG_TAG " vt_char_cols returns 0.\n");
-
-        continue;
-      }
-#endif
-
       col += vt_char_cols(line->chars + count);
     }
 
