@@ -202,7 +202,7 @@ typedef struct vt_parser {
   vt_read_buffer_t r_buf;
   vt_write_buffer_t w_buf;
 
-  vt_pty_ptr_t pty;
+  vt_pty_t *pty;
   vt_pty_hook_t pty_hook;
 
   vt_screen_t *screen;
@@ -268,6 +268,7 @@ typedef struct vt_parser {
   int8_t modify_cursor_keys;
   int8_t modify_function_keys;
   int8_t modify_other_keys;
+  u_char prev_local_echo_char;
 
   /* Used for non iso2022 encoding */
   ef_charset_t gl;
@@ -338,6 +339,8 @@ void vt_set_primary_da(char *da);
 
 void vt_set_secondary_da(char *da);
 
+void vt_set_local_echo_wait(u_int msec);
+
 #define vt_parser_init vt_config_proto_init
 
 void vt_parser_final(void);
@@ -351,9 +354,9 @@ vt_parser_t *vt_parser_new(vt_screen_t *screen, vt_termcap_ptr_t termcap,
                            int use_ansi_colors, vt_alt_color_mode_t alt_color_mode,
                            vt_cursor_style_t cursor_style, int ignore_broadcasted_chars);
 
-int vt_parser_delete(vt_parser_t *vt_parser);
+int vt_parser_destroy(vt_parser_t *vt_parser);
 
-void vt_parser_set_pty(vt_parser_t *vt_parser, vt_pty_ptr_t pty);
+void vt_parser_set_pty(vt_parser_t *vt_parser, vt_pty_t *pty);
 
 void vt_parser_set_xterm_listener(vt_parser_t *vt_parser,
                                         vt_xterm_event_listener_t *xterm_listener);
@@ -434,16 +437,16 @@ int vt_parser_is_broadcasting(vt_parser_t *vt_parser);
 
 int true_or_false(const char *str);
 
-int vt_parser_get_config(vt_parser_t *vt_parser, vt_pty_ptr_t output, char *key,
-                               int to_menu, int *flag);
+int vt_parser_get_config(vt_parser_t *vt_parser, vt_pty_t *output, char *key,
+                         int to_menu, int *flag);
 
 int vt_parser_set_config(vt_parser_t *vt_parser, char *key, char *val);
 
 int vt_parser_exec_cmd(vt_parser_t *vt_parser, char *cmd);
 
 void vt_parser_report_mouse_tracking(vt_parser_t *vt_parser, int col, int row,
-                                           int button, int is_released, int key_state,
-                                           int button_state);
+                                     int button, int is_released, int key_state,
+                                     int button_state);
 
 #define vt_parser_is_visible_cursor(vt_parser) ((vt_parser)->is_visible_cursor)
 
