@@ -55,7 +55,7 @@ int start_with_local_pty = 0;
 
 #ifdef OPEN_PTY_ASYNC
 
-static void pty_args_delete(pty_args_t *args) {
+static void pty_args_destroy(pty_args_t *args) {
   int count;
 
   free(args->cmd_path);
@@ -161,7 +161,7 @@ static void *
 #endif
     open_pty(void *p) {
   pty_args_t *args;
-  vt_pty_ptr_t pty;
+  vt_pty_t *pty;
 #ifdef USE_WIN32API
   static HANDLE mutex;
 
@@ -198,7 +198,7 @@ static void *
     args->term->return_special_pid = 0;
   }
 
-  pty_args_delete(args);
+  pty_args_destroy(args);
 
 #ifdef USE_WIN32API
   ReleaseMutex(mutex);
@@ -297,11 +297,11 @@ vt_term_t *vt_term_new(const char *term_type, u_int cols, u_int rows, u_int tab_
 
 error:
   if (term->screen) {
-    vt_screen_delete(term->screen);
+    vt_screen_destroy(term->screen);
   }
 
   if (term->parser) {
-    vt_parser_delete(term->parser);
+    vt_parser_destroy(term->parser);
   }
 
   free(term);
@@ -309,7 +309,7 @@ error:
   return NULL;
 }
 
-void vt_term_delete(vt_term_t *term) {
+void vt_term_destroy(vt_term_t *term) {
 #ifndef NO_IMAGE
   if (vt_term_pty_closed_event) {
     (*vt_term_pty_closed_event)(term);
@@ -319,7 +319,7 @@ void vt_term_delete(vt_term_t *term) {
   free(term->user_data);
 
   if (term->pty) {
-    vt_pty_delete(term->pty);
+    vt_pty_destroy(term->pty);
   } else if (term->pty_listener) {
     (*term->pty_listener->closed)(term->pty_listener->self);
   }
@@ -328,22 +328,22 @@ void vt_term_delete(vt_term_t *term) {
   free(term->icon_path);
   free(term->bidi_separators);
 
-  vt_screen_delete(term->screen);
-  vt_parser_delete(term->parser);
+  vt_screen_destroy(term->screen);
+  vt_parser_destroy(term->parser);
 
   free(term);
 }
 
 void vt_term_zombie(vt_term_t *term) {
   if (term->pty) {
-    vt_pty_ptr_t pty;
+    vt_pty_t *pty;
 
     pty = term->pty;
 
-    /* Should be NULL because vt_pty_delete calls term->pty_listener->closed. */
+    /* Should be NULL because vt_pty_destroy calls term->pty_listener->closed. */
     term->pty = NULL;
 
-    vt_pty_delete(pty);
+    vt_pty_destroy(pty);
   }
 #ifdef DEBUG
   else {
@@ -358,12 +358,17 @@ int vt_term_open_pty(vt_term_t *term, const char *cmd_path, char **argv, char **
                      const char *privkey, u_int width_pix, u_int height_pix) {
   if (!term->pty) {
 #ifdef OPEN_PTY_ASYNC
+    char *host_dup;
     char *user;
     char *server;
     char *port;
 
-    if (pass && bl_parse_uri(NULL, &user, &server, &port, NULL, NULL, bl_str_alloca_dup(host)) &&
-        !vt_search_ssh_session(server, port, user)) {
+    if (pass && (host_dup = alloca(strlen(host) + 1)) &&
+        bl_parse_uri(NULL, &user, &server, &port, NULL, NULL, strcpy(host_dup, host))
+#ifdef USE_LIBSSH2
+        && !vt_search_ssh_session(server, port, user)
+#endif
+        ) {
       pty_args_t *args;
 
       if (!(args = pty_args_new(term, cmd_path, argv, env, host, work_dir, pass, pubkey, privkey,
@@ -398,7 +403,7 @@ int vt_term_open_pty(vt_term_t *term, const char *cmd_path, char **argv, char **
     } else
 #endif /* OPEN_PTY_ASYNC */
     {
-      vt_pty_ptr_t pty;
+      vt_pty_t *pty;
 
       if (!(pty = vt_pty_new(cmd_path, argv, env, host, work_dir, pass, pubkey, privkey,
                              vt_screen_get_logical_cols(term->screen),
@@ -419,8 +424,7 @@ int vt_term_open_pty(vt_term_t *term, const char *cmd_path, char **argv, char **
   return 1;
 }
 
-int vt_term_plug_pty(vt_term_t *term, vt_pty_ptr_t pty /* Not NULL */
-                     ) {
+int vt_term_plug_pty(vt_term_t *term, vt_pty_t *pty /* Not NULL */) {
   if (!term->pty) {
     if (term->pty_listener) {
       vt_pty_set_listener(pty, term->pty_listener);
@@ -538,6 +542,14 @@ pid_t vt_term_get_child_pid(vt_term_t *term) {
   return vt_pty_get_pid(term->pty);
 }
 
+pid_t vt_term_get_pty_mode(vt_term_t *term) {
+  if (term->pty == NULL) {
+    return PTY_NONE;
+  }
+
+  return vt_pty_get_mode(term->pty);
+}
+
 size_t vt_term_write(vt_term_t *term, u_char *buf, size_t len) {
   if (term->pty == NULL) {
     return 0;
@@ -753,7 +765,7 @@ int vt_term_update_special_visual(vt_term_t *term) {
   int had_logvis = 0;
   int has_logvis = 0;
 
-  had_logvis = vt_screen_delete_logical_visual(term->screen);
+  had_logvis = vt_screen_destroy_logical_visual(term->screen);
 
   if (term->use_dynamic_comb) {
     if ((logvis = vt_logvis_comb_new())) {
@@ -771,7 +783,7 @@ int vt_term_update_special_visual(vt_term_t *term) {
         bl_warn_printf(BL_DEBUG_TAG " vt_screen_add_logical_visual failed.\n");
 #endif
 
-        (*logvis->delete)(logvis);
+        (*logvis->destroy)(logvis);
       }
     }
 #ifdef DEBUG
@@ -793,7 +805,7 @@ int vt_term_update_special_visual(vt_term_t *term) {
         bl_warn_printf(BL_DEBUG_TAG " vt_screen_add_logical_visual failed.\n");
 #endif
 
-        (*logvis->delete)(logvis);
+        (*logvis->destroy)(logvis);
       }
     }
 #ifdef DEBUG
@@ -812,7 +824,7 @@ int vt_term_update_special_visual(vt_term_t *term) {
         bl_warn_printf(BL_DEBUG_TAG " vt_screen_add_logical_visual failed.\n");
 #endif
 
-        (*logvis->delete)(logvis);
+        (*logvis->destroy)(logvis);
       }
     }
 #ifdef DEBUG
