@@ -19,7 +19,7 @@ typedef struct vt_term {
   /*
    * private
    */
-  vt_pty_ptr_t pty;
+  vt_pty_t *pty;
   vt_pty_event_listener_t *pty_listener; /* pool until pty opened. */
 
   vt_parser_t *parser;
@@ -71,7 +71,7 @@ vt_term_t *vt_term_new(const char *term_type, u_int cols, u_int rows, u_int tab_
                        int use_ansi_colors, vt_alt_color_mode_t alt_color_mode, int use_ot_layout,
                        vt_cursor_style_t cursor_style, int ignore_broadcasted_chars);
 
-void vt_term_delete(vt_term_t *term);
+void vt_term_destroy(vt_term_t *term);
 
 void vt_term_zombie(vt_term_t *term);
 
@@ -79,7 +79,7 @@ int vt_term_open_pty(vt_term_t *term, const char *cmd_path, char **argv, char **
                      const char *host, const char *work_dir, const char *pass, const char *pubkey,
                      const char *privkey, u_int width_pix, u_int height_pix);
 
-int vt_term_plug_pty(vt_term_t *term, vt_pty_ptr_t pty);
+int vt_term_plug_pty(vt_term_t *term, vt_pty_t *pty);
 
 #define vt_term_pty_is_opened(term) ((term)->pty != NULL)
 
@@ -146,6 +146,8 @@ char *vt_term_get_slave_name(vt_term_t *term);
 
 pid_t vt_term_get_child_pid(vt_term_t *term);
 
+pid_t vt_term_get_pty_mode(vt_term_t *term);
+
 size_t vt_term_write(vt_term_t *term, u_char *buf, size_t len);
 
 #define vt_term_write_modified_key(term, key, modcode) \
